
import threading
import socket
import cPickle
import socket # For gethostbyaddr()
import SocketServer
import sys
import traceback
import copy
import os

# Dictionary Emulator Object Database
#
# After being constructed, DictDBLocal and DictDBClient objects
# work like dictionary objects, but with the following additional methods:
#
# standby() optimizes index file to incorporate changes and purges
#           indexes from memory (indexes will be reread upon next call, if any)
#
# purge() repacks the database and index files to eliminate wasted space
#
# get_info() returns DictDBInfo object containing database statistics
#
# shutdown() terminates the database server (only useful for remote clients)
#
# reset() clears database

# PART 1
# Database Engine, with a thread-safe API

class DictDBInfo:
   def __init__(self):
      self.used = 0   # bytes containing data
      self.wasted = 0 # bytes wasted (from deleted/overwitten records)
      
class DictDBLocal:

   __magic__ = r'*8#~'  # 32-bit record stamp for record
   __delete_magic__ = '*8#@'  # 32-bit record stamp for deleted record
   

   def __init__(self,prefix,bin=1,read_only=0):

      # store important information
      self.index_file = prefix + ".dbi"  # database information (can be reconstructed from .dbf) 
      self.data_file = prefix + ".dbf"   # database data 
      self.bin = bin
      self.changed = 0
      self.read_only = read_only
      
      # create lock
      self.lock = threading.RLock() # NOTE: recursive for convenience
      
      # restore indexed into memory
      self._restore()


   def get_info(self):
      result = None

      # restore dictionary (if nec.)
      if not hasattr(self,'rec'):
         self._restore()
      try:
         self.lock.acquire()

         # generate independent copy for calling thread
         result = copy.deepcopy(self.info)
      finally:
         self.lock.release()
      return result


   def has_key(self,key):
      result = None

      # restore dictionary (if nec.)      
      if not hasattr(self,'rec'):
         self._restore()
      try:
         self.lock.acquire()
         result = self.rec.has_key(key)
      finally:
         self.lock.release()
      return result


   def keys(self):
      result = None

      # restore dictionary (if nec.)      
      if not hasattr(self,'rec'):
         self._restore()
      try:
         self.lock.acquire()
         
         # get list of keys (thread safe...result is a new object)
         result = self.rec.keys()
      finally:
         self.lock.release()
      return result


   def __delitem__(self,key):
      result = None
      # restore dictionary (if nec.)      
      if not hasattr(self,'rec'):
         self._restore()
      try:
         self.lock.acquire()
         if self.rec.has_key(key):
            
            # account for space
            if self.rec.has_key(key):
               self.info.wasted = self.info.wasted + self.rec[key][1]

            # delete record
            del self.rec[key]

            if not self.read_only:
               # write delete magic to data file for recovery
               f=open(self.data_file,'ab')
               f.write(self.__delete_magic__) 
               cPickle.dump(key,f,self.bin) 
               f.close()

               # write blank index to index file
               f=open(self.index_file,'ab')
               cPickle.dump((key,None),f,self.bin)
               f.close()
            
            # note change
            self.changed = 1
         else:
            raise KeyError(key)
      finally:
         self.lock.release()
      return result


   def standby(self):
      result = None

      # only do something if dictionary is already in RAM
      if hasattr(self,'rec') and not self.read_only:
         try:
            self.lock.acquire()
            
            # write index file
            f=open(self.index_file,'wb')
            cPickle.dump(self.info,f,self.bin)
            cPickle.dump(self.rec,f,self.bin)
            f.close()
            
            # free memory
            del self.rec
            del self.info
         finally:
            self.lock.release()
      return result

   def reset(self):
      result = None
      try:
         self.lock.acquire()

         if not self.read_only:
         # make sure files exist and are writable

            f = open(self.index_file,'wb')
            f.close()

            f = open(self.data_file,'wb')
            f.close()

         # new database information
         
         self.info = DictDBInfo()
         self.rec = {}
         self.changed = 1
         self.standby() # write out blank indexes (important)
         
      finally:
         self.lock.release()
      return result
      
   def purge(self):
      result = None
      # restore dictionary (if nec.)
      if not hasattr(self,'rec'):
         self._restore()
      try:
         self.lock.acquire()
         if not self.read_only:
            # open source and temporary data files
            tmp_data = self.data_file + '_tmp'
            f = open(self.data_file,'rb')
            g = open(tmp_data,'wb')
            rec = self.rec
            used = 0

            # iterate through records, copying only those which are extant
            for key in self.rec.keys():
               f.seek(rec[key][0])
               g.write(self.__magic__)
               cPickle.dump(key,g,self.bin);
               start = g.tell()
               g.write(f.read(rec[key][1]))
               used = used + rec[key][1]
               rec[key]=(start,rec[key][1]) # generate new record entry
            f.close()
            g.close()

            # now perform the switch-over 
            os.unlink(self.index_file) # delete old index file...         
            os.unlink(self.data_file)  # delete old data file
            os.rename(tmp_data,self.data_file) # move new data file over old

            # update database information
            self.info.used = used
            self.info.wasted = 0

            # write new index and information file
            f=open(self.index_file,'wb')
            cPickle.dump(self.info,f,self.bin)
            cPickle.dump(self.rec,f,self.bin)
            f.close()
      finally:
         self.lock.release()
      return result

   def shutdown(self): # dummy
      return None

   def __getitem__(self,key): # get with object
      result = None
      # restore dictionary (if nec.)
      if not hasattr(self,'rec'):
         self._restore()
      try:
         self.lock.acquire()
         if self.rec.has_key(key):

            # locate and retrieve object
            f=open(self.data_file,'rb')
            f.seek(self.rec[key][0])
            result = cPickle.load(f)
            f.close()
         else:
            raise KeyError(key)
      finally:
         self.lock.release()
      return result


   def _get(self,key): # get with string (for remote connections)
      result = None
      # restore dictionary (if nec.)
      if not hasattr(self,'rec'):
         self._restore()
      try:
         self.lock.acquire()
         if self.rec.has_key(key):

            # locate and retrieve string            
            f=open(self.data_file,'rb')
            f.seek(self.rec[key][0])
            result = f.read(self.rec[key][1])
            f.close()
      finally:
         self.lock.release()
      return result


   def __setitem__(self,key,object):
      result = None
      # restore dictionary (if nec.)
      if not hasattr(self,'rec'):
         self._restore()
      try:
         self.lock.acquire()
         
         # append data onto data file
         f=open(self.data_file,'ab')
         f.write(self.__magic__) # for recovery
         cPickle.dump(key,f,self.bin) # for recovery
         start = f.tell()
         cPickle.dump(object,f,self.bin)
         record_info = (start,f.tell()-start)
         f.close()
         
         # account for space (if replacing)
         if self.rec.has_key(key):
            self.info.wasted = self.info.wasted + self.rec[key][1]

         # update record info
         self.rec[key] = record_info

         # account for space
         self.info.used = self.info.used + record_info[1]
         
         # append new record  onto index file
         f=open(self.index_file,'ab')
         cPickle.dump((key,record_info),f,self.bin)
         f.close()
         
         # note change
         self.changed = 1
      finally:
         self.lock.release()
      return result


   def _set(self,key,data_string): # set with string (for remote connections)
      result = None
      # restore dictionary (if nec.)
      if not hasattr(self,'rec'):
         self._restore()
      try:
         self.lock.acquire()
         self.changed = 1

         if not self.read_only:
            # append data onto data file         
            f=open(self.data_file,'ab')
            f.write(self.__magic__) # for recovery
            cPickle.dump(key,f,self.bin) # for recovery
            start = f.tell()
            f.write(data_string)
            record_info = (start,f.tell()-start)
            f.close()

         # account for space (if replacing)       
         if self.rec.has_key(key):
            self.info.wasted = self.info.wasted + self.rec[key][1]

         # update record info            
         self.rec[key] = record_info

         # account for space
         self.info.used = self.info.used + record_info[1]

         if not self.read_only:
            # append new record onto index file
            f=open(self.index_file,'ab')
            cPickle.dump((key,record_info),f,self.bin)
            f.close()
      finally:
         self.lock.release()
      return result


   def _recover(self): # rebuild index from data file
      result = None
      # need to write recovery routine...
      self.info = DictDBInfo()
      self.rec = {}
      try:
         try:
            self.lock.acquire()
            f=open(self.data_file,'rb')
            
            # find length of file
            f.seek(0,2)
            eof = f.tell()
            f.seek(0,0)

            # create locals for better performance
            rec = self.rec
            self_info = self.info
            while f.tell()!=eof:
               chk = f.read(4)
               if chk==self.__magic__: # recover extant record
                  key = cPickle.load(f)
                  start = f.tell()
                  data = cPickle.load(f)
                  record_info = (start,f.tell()-start)
                  
                  # account for space (if replacing)
                  if rec.has_key(key):
                     self_info.wasted = self_info.wasted + rec[key][1]                  
                  rec[key] = record_info

                  # account for psace
                  self_info.used = self_info.used + record_info[1]
               elif chk==self.__delete_magic__: # delete already recovered record
                  key = cPickle.load(f)
                  if rec.has_key(key):
                     # account for space
                     self_info.wasted = self_info.wasted + rec[key][1]
                     del rec[key]
               else:
                  raise RuntimeError('Bad Magic')
         except:
            print " dictdb error: database recovery failed."
            traceback.print_exc()
            sys.exit(1)
         if not self.read_only:
            # write recovered indexes to a new index file
            f = open(self.index_file,'wb')
            cPickle.dump(self.info,f,self.bin)
            cPickle.dump(self.rec,f,self.bin)
            f.close()
      finally:
         self.lock.release()
      return result
   

   def _restore(self):
      result = None
      try:
         self.lock.acquire()

         # make sure files exist and are writable

         f = open(self.index_file,'ab')
         f.close()

         f = open(self.data_file,'ab')
         f.close()

         # read current indexes

         f = open(self.index_file,'rb')

         # find length of file

         f.seek(0,2)
         eof = f.tell()
         f.seek(0,0)

         # start reading and appending recent changes (if any)

         try:
            self.info = cPickle.load(f)
            self.rec = cPickle.load(f)

            # use locals for better performance 
            rec = self.rec
            cPickle_load = cPickle.load
            self_info = self.info
            
            while f.tell()!=eof:
               key, info = cPickle_load(f)
               if info!=None:

                  # account for space (if replacing)
                  if rec.has_key(key):
                     self_info.wasted = self_info.wasted + rec[key][1]

                  # set record infor
                  rec[key] = info

                  # account for space
                  self_info.used = self_info.used + info[1]
               else:
                  # account for space (if deleting object)
                  self_info.wasted = self_info.wasted + rec[key][1]
                  del rec[key]
            f.close()
         except EOFError:
            # if error occurs when reading indexes, then do a datafile-based recovery
            f.close()
            self._recover()
      finally:
         self.lock.release()
      return result

# PART 2
# Database client

class DictDBClient:

   def __init__(self,host='localhost',port=8000):
      self.host=host
      self.port=port
      self.sock = None

   def _remote_call(self,meth,args,kwds):
      result = None
      if self.sock == None:
         self.sock=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
         self.sock.connect((self.host,self.port))
         self.send = self.sock.makefile('w')
         self.recv = self.sock.makefile('r')
      cPickle.dump(meth,self.send,1) # binary by default
      cPickle.dump(args,self.send,1)
      cPickle.dump(kwds,self.send,1)
      self.send.flush()
      result = cPickle.load(self.recv)
      return result

   def __getitem__(self,key):
      if self._remote_call('has_key',(key,),{}):
         return cPickle.loads(self._remote_call('_get',(key,),{}))
      else:
         raise KeyError(key)

   def __setitem__(self,key,object):
      self.changed = 1
      return self._remote_call('_set',(key,cPickle.dumps(object)),{})


   def __delitem__(self,key):
      if self._remote_call('has_key',(key,),{}):
         return self._remote_call('__delitem__',(key,),{})
      else:
         raise KeyError(key)

   def standby(self):
      return self._remote_call('standby',(),{})

   def shutdown(self): 
      try:
         # multiple connections are sometimes required...         
         self._remote_call('shutdown',(),{})
         self.sock=None
         self._remote_call('shutdown',(),{})
         self.sock=None
         self._remote_call('shutdown',(),{})
         self.sock=None
         self._remote_call('shutdown',(),{})
      except:
         pass
      return None

   def purge(self):
      return self._remote_call('purge',(),{})

   def reset(self):
      return self._remote_call('reset',(),{})

   def keys(self):
      return self._remote_call('keys',(),{})

   def has_key(self,key):
      return self._remote_call('has_key',(key,),{})

   def get_info(self):
      return self._remote_call('get_info',(),{})

# PART 3
# Database Socket Server

class DictDBServer:
   def __init__(self,prefix,port=''):

      sys.setcheckinterval(0)
      
      server_address = ('', port)

      ddbs = _DictDBServer(server_address, DictDBRequestHandler)  

      # assign dict database to this server
      ddbs.dictdb = DictDBLocal(prefix)

      # now serve requests forever
      ddbs.keep_alive = 1
      while ddbs.keep_alive:
         ddbs.handle_request()
      
class _DictDBServer(SocketServer.ThreadingTCPServer):

    def server_bind(self):
        """Override server_bind to store the server name."""
        SocketServer.ThreadingTCPServer.server_bind(self)
        host, port = self.socket.getsockname()
        if not host or host == '0.0.0.0':
            host = socket.gethostname()
        hostname, hostnames, hostaddrs = socket.gethostbyaddr(host)
        if '.' not in hostname:
            for host in hostnames:
                if '.' in host:
                    hostname = host
                    break
        self.server_name = hostname
        self.server_port = port

class DictDBRequestHandler(SocketServer.StreamRequestHandler):

    def handle(self):
       while self.server.keep_alive:
          # get method name from client

          try:
             method = cPickle.load(self.rfile)
          except EOFError,socket.error:
             break

          if method == 'shutdown':
             self.server.keep_alive = 0
             
          # get arguments from client
          args = cPickle.load(self.rfile)
          kw = cPickle.load(self.rfile)

          # get method pointer
          meth_obj = getattr(self.server.dictdb,method)
#          print method,args,kw

          # call method and return result
          cPickle.dump(apply(meth_obj,args,kw),self.wfile,1) # binary by default
          self.wfile.flush()
          
def server_test(port = 8000,prefix='test_dictdb'):
   print 'Testing DictDBServer on port',str(port)
   fp = DictDBServer(prefix,port=port) # socket servers don't terminate

def client_test(host,port=8000):

   print 'Testing Client with Server on port',str(port)

   tdb = DictDBClient(port=port)
   tdb['test']='hello'
   print tdb['test']
   try:
      print tdb['nonexistent']
   except KeyError:
      print " key error 1 as expected"
   try:
      del tdb['nonexistent']
   except KeyError:
      print " key error 2 as expected"
   tdb['extra'] = 'hi'
   print tdb.has_key('extra')
   print tdb.keys()

if __name__=='__main__':
   import os
   
   print '***Testing DictDB***:'

   ddb = DictDBLocal('test_dictdb')
   print ddb.keys()
   ddb['test']='some data object'
   print ddb['test']
   ddb['another']='another data object'
   print ddb['another']
   ddb['test']='some updated data object'   
   print ddb['test']
   del ddb

   ddb = DictDBLocal('test_dictdb')
   ddb['whoa']='whoa data object'
   print ddb['test'] 
   ddb['dude']='dude data object'
   print ddb['dude']
   ddb['number'] = 9999
   print ddb['number']
   ddb.standby()
   del ddb

   ddb = DictDBLocal('test_dictdb')
   print 'Current keys:'+str(ddb.keys())

   print ddb['test']
   print ddb['another']

   print '***Testing raw string methods:'
   print cPickle.loads(ddb._get('test'))
   print cPickle.loads(ddb._get('another'))

   ddb._set('some_key',cPickle.dumps("raw test1"))
   print ddb['some_key']

   info = ddb.get_info()
   print info.used,info.wasted,info.used-info.wasted
   del ddb
   
   os.unlink("test_dictdb.dbi")
   ddb = DictDBLocal('test_dictdb')
   info = ddb.get_info()
   print info.used,info.wasted,info.used-info.wasted
   del ddb

   os.unlink("test_dictdb.dbi")
   ddb = DictDBLocal('test_dictdb')
   info = ddb.get_info()
   print info.used,info.wasted,info.used-info.wasted
   ddb.standby()
   del ddb

   ddb = DictDBLocal('test_dictdb')
   info = ddb.get_info()
   print info.used,info.wasted,info.used-info.wasted
   del ddb

   ddb = DictDBLocal('test_dictdb')
   ddb.standby()
   ddb['reactivated'] = 1234
   print ddb.has_key('reactivated')
   ddb.standby()
   ddb['reactivated'] = 495
   print ddb['reactivated']
   try:
      print ddb['nonexistent']
   except KeyError:
      print " key error 1 as expected"
   try:
      del ddb['nonexistent']
   except KeyError:
      print " key error 2 as expected"
   
   print "purge test:"
   info = ddb.get_info()
   print info.used,info.wasted,info.used-info.wasted
   ddb.purge()
   info = ddb.get_info()
   print info.used,info.wasted,info.used-info.wasted
   del ddb

   ddb = DictDBLocal('test_dictdb')
   info = ddb.get_info()
   print info.used,info.wasted,info.used-info.wasted

   ddb['hello']=10
   ddb[123] = 'bacon'
   ddb['green']= 'color'

   print ddb['hello']

   print ddb.keys()

   print ddb.has_key('green')
   print ddb.has_key('blue')

   del ddb['hello']

   try:
      print ddb['hello']       
   except KeyError:
      print ' got expected key error 1'

   try:
      del ddb['hello']       
   except KeyError:
      print ' got expected key error 2'

   info = ddb.get_info()
   print info.used,info.wasted,info.used-info.wasted
   ddb.purge()
   info = ddb.get_info()
   print info.used,info.wasted,info.used-info.wasted
   print ddb['green']

   import whrandom
   
   ddc=DictDBLocal('test_dictdb')

   print "loading..."

   lst = []
   for a in xrange(1,250):
      ddc[a]=str(a)
      lst.append([whrandom.random(),a])
      ddc[str(a)] = a

   info = ddc.get_info()
   print 'used:',info.used,'wasted:',info.wasted,'extant:',info.used-info.wasted

   print 'verifying...'
   for a in xrange(1,250):
      if ddc[a]!=str(a):
         raise RuntimeError

   for a in xrange(1,250):
      del ddc[str(a)]

   info = ddc.get_info()
   print 'used:',info.used,'wasted:',info.wasted,'extant:',info.used-info.wasted

   print 'verifying...'
   for a in xrange(1,250):
      if ddc[a]!=str(a):
         raise RuntimeError

   del ddc
   ddc=DictDBLocal('test_dictdb')

   info = ddc.get_info()
   print 'used:',info.used,'wasted:',info.wasted,'extant:',info.used-info.wasted

   print 'verifying...'
   for a in xrange(1,250):
      if ddc[a]!=str(a):
         raise RuntimeError

   ddc.standby()

   del ddc

   os.unlink("test_dictdb.dbi")
   print "unlink test"
   
   ddc=DictDBLocal('test_dictdb')

   info = ddc.get_info()
   print 'used:',info.used,'wasted:',info.wasted,'extant:',info.used-info.wasted

   print 'verifying...'
   for a in xrange(1,250):
      if ddc[a]!=str(a):
         raise RuntimeError

   ddc.purge()

   info = ddc.get_info()
   print 'used:',info.used,'wasted:',info.wasted,'extant:',info.used-info.wasted

   print 'verifying...'
   for a in xrange(1,250):
      if ddc[a]!=str(a):
         raise RuntimeError

   info = ddc.get_info()
   print 'used:',info.used,'wasted:',info.wasted,'extant:',info.used-info.wasted

   lst.sort()

   for a in lst:
      del ddc[a[1]]

   info = ddc.get_info()
   print 'used:',info.used,'wasted:',info.wasted,'extant:',info.used-info.wasted

   del ddc
   ddc=DictDBLocal('test_dictdb')

   info = ddc.get_info()
   print 'used:',info.used,'wasted:',info.wasted,'extant:',info.used-info.wasted

   ddc.purge()

   info = ddc.get_info()
   print 'used:',info.used,'wasted:',info.wasted,'extant:',info.used-info.wasted

   del ddc
   ddc=DictDBLocal('test_dictdb')

   info = ddc.get_info()
   print 'used:',info.used,'wasted:',info.wasted,'extant:',info.used-info.wasted

   print ddc.keys()
   
   ddc.reset()
   
   del ddc
   ddc=DictDBLocal('test_dictdb')

   info = ddc.get_info()
   print 'used:',info.used,'wasted:',info.wasted,'extant:',info.used-info.wasted


   



from chempy.models import Indexed
from chempy import Storage,Atom,Bond

import string

class CC1(Storage): # ChemDraw3D 5.0 std., cartesian coordinates

   def fromList(self,molList):

      model = Indexed()

      # read header information
      nAtom = int(molList[0][0:3])

      # read atoms and bonds
      id_dict = {}
      irec = 1
      cnt = 0
      for a in range(nAtom):
         at = Atom()
         at.index = cnt
         id_dict[string.strip(molList[irec][3:8])] = at.index
         at.coord = [float(molList[irec][8:20]), 
            float(molList[irec][20:32]),float(molList[irec][32:44])]
         at.symbol = string.strip(molList[irec][0:3])
         at.numeric_type = int(molList[irec][44:49])
         lst = string.split(string.strip(molList[irec][49:]))
         at.bonds = lst
         irec = irec + 1
         cnt = cnt + 1
         model.atom.append(at)
         
      # interpret bonds
      cnt = 0
      for a in model.atom:
         lst = a.bonds
         del a.bonds
         for b in lst:
            if a.index<id_dict[b]:
               bnd = Bond()
               bnd.index = [ a.index,id_dict[b]]
               model.bond.append(bnd)

      # obtain formal charges from M  CHG record
      return model

#------------------------------------------------------------------------------
   def toList(self,model):

      return []

      # not implemented yet




#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------

import selector

import cmd

from cmd import _cmd,lock,unlock,Shortcut,QuietException
from cmd import _feedback,fb_module,fb_mask

def deselect():
   '''
DESCRIPTION
  
   "deselect" disables any and all visible selections
 
USAGE
 
   deselect
 
PYMOL API
  
   cmd.deselect()
   '''
   arg = cmd.get_names("selections")
   for a in arg:
      cmd.disable(a)

def select(name,selection="",quiet=0,show=0):
   '''
DESCRIPTION
  
   "select" creates a named selection from an atom selection.
 
USAGE
 
   select (selection)
   select name, (selection)
   select name = (selection)            # (DEPRECATED)
 
PYMOL API
  
   cmd.select(string name, string selection)
 
EXAMPLES 

   select near , (ll expand 8)
   select near , (ll expand 8)
   select bb, (name ca,n,c,o )

NOTES

   'help selections' for more information about selections.
   '''   
   try:
      lock()
      if selection=="":
         sel_cnt = _cmd.get("sel_counter") + 1.0
         _cmd.legacy_set("sel_counter","%1.0f" % sel_cnt)
         selection = name
         name = "sel%02.0f" % sel_cnt
      else:
         name = name
      # preprocess selection (note: inside TRY)
      selection = selector.process(selection)
      #
      r = _cmd.select(str(name),str(selection),int(quiet))
      if r and show:
         r = _cmd.onoff(str(name),1);
   finally:
      unlock()
   return r

def indicate(selection="(all)"):
   '''
DESCRIPTION
  
   "indicate" shows a visual representation of an atom selection.
 
USAGE
 
   indicate (selection)
 
PYMOL API
  
   cmd.count(string selection)
 
   '''
   # preprocess selection
   selection = selector.process(selection)
   #      
   try:
      lock()   
      r = _cmd.select("indicate","("+str(selection)+")",1)
      cmd.enable("indicate")
   finally:
      unlock()
   return r



#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------

import os
import thread
import selector
import re

import pymol
import cmd
from cmd import _cmd,lock,unlock,Shortcut,QuietException
from chempy import io
from cmd import _feedback,fb_module,fb_mask

def png(filename):
   '''
DESCRIPTION
  
   "png" writes a png format image file of the current image to disk.
   
USAGE
 
   png filename
 
PYMOL API
 
   cmd.png( string file )
   '''
   if thread.get_ident() ==pymol.glutThread:
      r = cmd._png(str(filename))
   else:
      r = _cmd.do("cmd._png('"+str(filename)+"')")
   return r

def export_coords(obj,state): # experimental
   r = None
   try:
      lock()   
      r = _cmd.export_coords(str(obj),int(state)-1)
   finally:
      unlock()
   return r

def multisave(filename,object,state=0):
   r = 1
   try:
      lock()
      _cmd.multisave(str(filename),str(object),int(state)-1,0)
   finally:
      unlock()
   return r

def save(filename,selection='(all)',state=0,format=''):
   '''
DESCRIPTION
  
   "save" writes selected atoms to a file.  The file format is
   autodetected if the extesion is ".pdb" or ".pkl"
 
USAGE
 
   save file [,(selection) [,state [,format]] ]
 
PYMOL API
  
   cmd.save(file, selection, state, format)

SEE ALSO

   load, get_model
   '''
   # preprocess selection
   selection = selector.process(selection)
   #   
   r = 1
   if format=='':
      format = 'pdb'
      if re.search("\.pdb$|\.ent$",filename):
         format = 'pdb'
      elif re.search("\.mol$",filename):
         format = 'mol'
      elif re.search("\.pkl$",filename):
         format = 'pkl'
      elif re.search("\.pkl$",filename):
         format = 'pkla'
      elif re.search("\.mmd$",filename):
         format = 'mmod'
      elif re.search("\.mmod$",filename):
         format = 'mmod'
      elif re.search("\.pmo$",filename):
         format = 'pmo'
   else:
      format = str(format)
   filename = os.path.expanduser(filename)
   filename = os.path.expandvars(filename)
   if format=='pdb':
      f=open(filename,"w")
      if f:
         try:
            lock()
            st = _cmd.get_pdb("("+str(selection)+")",int(state)-1)
         finally:
            unlock()
            f.write(st)
            f.close()
         r = None
         print " Save: wrote \""+filename+"\"."
   elif format=='pkl': # python binary
      io.pkl.toFile(cmd.get_model(selection,state),filename)
      print " Save: wrote \""+filename+"\"."
   elif format=='pkla': # ascii override
      io.pkl.toFile(cmd.get_model(selection),filename,bin=0)
      print " Save: wrote \""+filename+"\"."
   elif format=='mmod': # macromodel
      io.mmd.toFile(cmd.get_model(selection),filename)
      print " Save: wrote \""+filename+"\"."
   elif format=='mol': 
      io.mol.toFile(cmd.get_model(selection),filename)
      print " Save: wrote \""+filename+"\"."
   return r





import os
import shutil
import glob
import re
import string
import sys
import time

from chempy import feedback

# "do" is the preferred command for running tinker

def do(command,in_prefix,run_prefix,out_prefix,tokens,capture=None):
   if feedback['tinker']:
      print " "+str(__name__)+': creating temporary files "%s.*"' % (run_prefix)
      print " "+str(__name__)+': launching %s...' % command
      c = 1
      for a in tokens:
         print " "+str(__name__)+': input %d = %s' % (c,a)
         c = c + 1
      if hasattr(sys.stdout,"flush"):
         sys.stdout.flush()
   for a in glob.glob(run_prefix+".*"):
      os.unlink(a)
   for a in glob.glob(out_prefix+".*"):
      os.unlink(a)
   for src in glob.glob(in_prefix+".*"):
      dst = string.split(src,'.')
      dst = run_prefix+'.'+dst[len(dst)-1]
      shutil.copyfile(src,dst)
   if capture==1:
      pipe = os.popen(bin_path+command+"> "+run_prefix+".out","w")      
   elif capture==2:
      pipe = os.popen(bin_path+command+" | tee "+run_prefix+".out","w")      
   else:
      pipe = os.popen(bin_path+command,"w")
   if not pipe:
      print "Error: can't run tinker!!!"
      raise RunError
   for a in tokens:
      pipe.write(a+"\n")
   pipe.close()
# NFS workaround (flushes the directory cache so that glob will work)
   try: os.unlink(".sync")
   except: pass
   f = open(".sync",'w')
   f.close()
#
   for src in glob.glob(run_prefix+".*_2"):
      dst = string.replace(src,'_2','')
      if os.path.exists(dst):
         os.unlink(dst)
#      os.rename(src,dst)    rename can fail over NFS (remote action)
      shutil.copyfile(src,dst)
# sloppy workaround for buggy NFS on linux
      os.unlink(src)
   for src in glob.glob(run_prefix+".*"):
      dst = string.split(src,'.')
      dst = out_prefix+'.'+dst[len(dst)-1]
      if os.path.exists(dst):
         os.unlink(dst)
#      os.rename(src,dst)    rename can fail over NFS (remote action)  
      shutil.copy(src,dst)
      os.unlink(src)
   for a in glob.glob(in_prefix+".*"):
      os.unlink(a)
   if feedback['tinker']:
      print " "+str(__name__)+': %s job complete. ' % command
      print " "+str(__name__)+': creating output files "%s.*"' % (out_prefix)

#  DEPRECATED

prefix = "tinker_run"

def run(command,in_prefix,out_prefix,tokens,capture=None):
   if feedback['tinker']:
      print " "+str(__name__)+': creating temporary files "%s.*"' % (prefix)
      print " "+str(__name__)+': launching %s...' % command
      c = 1
      for a in tokens:
         print " "+str(__name__)+': input %d = %s' % (c,a)
         c = c + 1
      if hasattr(sys.stdout,"flush"):
         sys.stdout.flush()
   for a in glob.glob(prefix+".*"):
      os.unlink(a)
   for a in glob.glob(out_prefix+".*"):
      os.unlink(a)
   for src in glob.glob(in_prefix+".*"):
      dst = string.split(src,'.')
      dst = prefix+'.'+dst[len(dst)-1]
      shutil.copyfile(src,dst)
   if capture:
      pipe = os.popen(bin_path+command+"> "+out_prefix+".out","w")      
   else:
      pipe = os.popen(bin_path+command,"w")
   if not pipe:
      print "Error: can't run tinker!!!"
      raise RunError
   for a in tokens:
      pipe.write(a+"\n")
   pipe.close()
   for src in glob.glob(prefix+".*_2"):
      dst = string.replace(src,'_2','')
      if os.path.exists(dst):
         os.unlink(dst)
#      os.rename(src,dst)    rename can fail over NFS (remote action)  
      shutil.copy(src,dst)
      os.unlink(src)
   for src in glob.glob(prefix+".*"):
      dst = string.split(src,'.')
      dst = out_prefix+'.'+dst[len(dst)-1]
      if os.path.exists(dst):
         os.unlink(dst)
#      os.rename(src,dst)    rename can fail over NFS (remote action)  
      shutil.copy(src,dst)
   if feedback['tinker']:
      print " "+str(__name__)+': %s job complete. ' % command
      print " "+str(__name__)+': creating output files "%s.*"' % (out_prefix)

if os.environ.has_key('TINKER_PATH'):
   base = os.environ['TINKER_PATH']
   bin_path = base + '/bin/'
   params_path = base + '/params/'
elif os.environ.has_key('FREEMOL_ETC'):
   base = os.environ['FREEMOL_ETC'] + '/tinker'
   bin_path = base + '/bin/'
   params_path = base + '/params/'
else:
   base = ''
   bin_path = ''
   params_path = ''

if os.environ.has_key('PYMOL_PATH'):
   pymol_path = os.environ['PYMOL_PATH']
   test_path = pymol_path + '/modules/chempy/tinker/'
   if os.path.exists(test_path):
      params_path = test_path





from chempy import Storage

import pickle
import cPickle

class PKL(Storage):

   def fromFile(self,fname,**params):
      fp = open(fname,'rb')
      result = cPickle.load(fp)
      fp.close()
      return result

#---------------------------------------------------------------------------
   def toFile(self,indexed,fname,**params):
      fp = open(fname,'wb')
      if(not params.has_key('bin')):
         result = cPickle.dump(indexed,fp,1)
      else:
         result = cPickle.dump(indexed,fp,params['bin'])         
      fp.close()

#---------------------------------------------------------------------------
   def fromStream(self,fp,**params):
      try:
         return cPickle.load(fp)
      except EOFError:
         return None

#---------------------------------------------------------------------------
   def toStream(self,indexed,fp,**params):
      if(not params.has_key('bin')):
         result = cPickle.dump(indexed,fp,1)
      else:
         result = cPickle.dump(indexed,fp,params['bin'])         
 
#---------------------------------------------------------------------------
   def fromString(self,st):
      return cPickle.loads(st)

#---------------------------------------------------------------------------
   def toString(self,model):
      return cPickle.dumps(st)




#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------

# pmgui_tk 
# TKinter based gui for PyMol
# NOTE: must have treads support compiled into python to use this module
#
# **This is the only module which should be/need be imported by 
# **PyMol Programs

from PMGApp import *
import sys

if sys.platform=='win32':
   if sys.version[0:4]=='2.1 ':
      if not os.environ.has_key('TCL_LIBRARY'):
         os.environ['TCL_LIBRARY']='c:\\python21\\tcl\\tcl8.3'
            
def run():
   if not hasattr(sys,"argv"):
      sys.argv=["pymol"]
   PMGApp(balloon_state='both').run()

t = threading.Thread(target=run,args=())
t.setDaemon(1)
t.start()








#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------

import thread
import string
import re
import os
import time

import traceback

import cmd
import pymol
from cmd import _cmd,lock,unlock,Shortcut,QuietException
from cmd import _feedback,fb_module,fb_mask

def resume(fname):
   if os.path.exists(fname):
      if(re.search(r"\.py$|\.PY$|\.pym$|.PYM$",fname)):
         cmd.do("run %s"%fname)
      else:
         cmd.do("@%s"%fname)
   cmd.do("log_open %s,a"%fname)

def log_open(fname='log.pml',mode='w'):
   try:
      try:
         if pymol._log_file!=None:
            pymol._log_file.close()
      except:
         pass
      pymol._log_file = open(fname,mode)
      if _feedback(fb_module.cmd,fb_mask.details): # redundant
         if mode!='a':
            print " Cmd: logging to '%s'."%fname
         else:
            print " Cmd: appending to '%s'."%fname            
      if mode=='a':
         pymol._log_file.write("\n") # always start on a new line
      if(re.search(r"\.py$|\.PY$|\.pym$|.PYM$",fname)):
         cmd.set("logging",2,quiet=1)
      else:
         cmd.set("logging",1,quiet=1)
   except:
      print"Error: unable to open log file '%s'"%fname
      pymol._log_file = None
      cmd.set("logging",0,quiet=1)
      traceback.print_exc()
      
def log(text,alt_text=None):
   if pymol._log_file!=None:
      mode = cmd.get_setting_legacy("logging")
      if mode:
         if mode==1:
            pymol._log_file.write(text)
         elif mode==2:
            if alt_text!=None:
               pymol._log_file.write(alt_text)
            else:
               pymol._log_file.write("cmd.do('''%s''')\n"%string.strip(text))
         pymol._log_file.flush()
         
def log_close():
   if pymol._log_file!=None:
      pymol._log_file.close()
      cmd.set("logging",0,quiet=1)
      if _feedback(fb_module.cmd,fb_mask.details): # redundant
         print " Cmd: log closed."

def cls(): 
   '''
DESCRIPTION
 
   "cls" clears the output buffer.

USAGE
 
   cls
'''
   r = None
   try:
      lock()
      r = _cmd.cls()
   finally:
      unlock()
   return r

def splash():
   '''
DESCRIPTION
 
   "splash" shows the splash screen information.

USAGE

   splash
   '''
   if cmd.get_setting_legacy("internal_feedback")>0.1:
      cmd.set("text","1",quiet=1)
   print
   try:
      lock()
      r = _cmd.splash()
   finally:
      unlock()
   return r

def sync(timeout=1.0,poll=0.05):
   '''
DESCRIPTION

   "sync" is an API-only function which waits until all current
   commmands have been executed before returning.  A timeout
   can be used to insure that this command eventually returns.

PYMOL API

   cmd.sync(float timeout=1.0,float poll=0.05)

SEE ALSO

   frame
'''
   now = time.time()
   timeout = float(timeout)
   poll = float(poll)
   if _cmd.wait_queue(): # commands waiting to be executed?
      while 1:
         e = threading.Event() # using this for portable delay
         e.wait(poll)
         del e
         if not _cmd.wait_queue():
            break
         if (time.time()-now)>timeout:
            break
            

def do(commands):
   # WARNING: don't call this routine if you already have the API lock
   '''
DESCRIPTION
 
   "do" makes it possible for python programs to issue simple PyMOL
   commands as if they were entered on the command line.
    
PYMOL API
 
   cmd.do( commands )
 
USAGE (PYTHON)
 
   from pymol import cmd
   cmd.do("load file.pdb")
   '''
   lst = string.split(commands,"\n")   
   try:
      lock()
      r = None
      for a in lst:
         if(len(a)):
            r = _cmd.do(a)
   finally:
      unlock()
   return r


def quit():
   '''
DESCRIPTION
  
   "quit" terminates the program. 
   
USAGE
 
   quit
 
PYMOL API
 
   cmd.quit()
   '''
   if thread.get_ident() == pymol.glutThread:
      try: 
         lock()
         r = _cmd.quit()
      finally:
         unlock()
   else:
      try:
         lock()
         r = _cmd.do("_cmd.quit()")
      finally:
         unlock()
   return r

def delete(name):
   '''
DESCRIPTION
  
   "delete" removes an object or a selection. 
   
USAGE
 
   delete name  
   delete all   # deletes all objects

   name = name of object or selection
  
PYMOL API
 
   cmd.delete (string name = object-or-selection-name )

SEE ALSO

   remove
   '''
   try:
      lock()   
      r = _cmd.delete(str(name))
   finally:
      unlock()
   return r



import thread 
import threading 
import os
import sys
import time
import __main__


if hasattr(__main__,"pymol_argv"):
   pymol_argv = __main__.pymol_argv
else:
   pymol_argv = sys.argv

modules_path = os.environ['PYMOL_PATH']+'/modules'

if modules_path not in sys.path:
   sys.path.append(modules_path)

import pymol

pymol.invocation.parse_args(pymol_argv)

pymol.start_pymol()








# This wizard contributed by Ezequiel "Zac" Panepucci 011114
# modified by Warren L. DeLano

from pymol.wizard import Wizard
from pymol import cmd
import pymol

class Label(Wizard):

   atom=None
   messages=1
   labeling=1
   obj_name=None

   def get_prompt(self):
      self.prompt = []
      if (not self.messages):
         return None

      if (self.atom == None):
         self.prompt = ['Click atoms...']
      else:
         if self.atom.chain == '':
            self.prompt.append( '%s %s %s %s B = %.2f  XYZ = %.3f %.3f %.3f' %
                            (self.obj_name,
                             self.atom.resn,
                             self.atom.resi,
                             self.atom.name,
                             self.atom.b,
                             self.atom.coord[0],
                             self.atom.coord[1],
                             self.atom.coord[2]) )
         else:
            self.prompt.append('%s %s %s%s %s B = %.2f  XYZ = %.3f %.3f %.3f' %
                            (self.obj_name,
                             self.atom.resn,
                             self.atom.chain,
                             self.atom.resi,
                             self.atom.name,
                             self.atom.b,
                             self.atom.coord[0],
                             self.atom.coord[1],
                             self.atom.coord[2]) )

      return self.prompt

   def toggle_messages(self):
      self.messages = not self.messages

   def toggle_labeling(self):
      self.labeling = not self.labeling

   def get_panel(self):
      return [
         [ 1, 'Labeling',''],
         [ 2, 'Toggle add/erase','cmd.get_wizard().toggle_labeling()'],
         [ 2, 'Toggle messages','cmd.get_wizard().toggle_messages()'],
         [ 2, 'Clear All','cmd.label()'],
         [ 2, 'Done','cmd.set_wizard()'],
         ]

   def do_pick(self,bondFlag):
      self.obj_name = None

#      if 'pk1' in cmd.get_names('selections'):
      if cmd.count_atoms('pk1',1):
         self.obj_name = cmd.identify('pk1',1)[0][0]

      model = cmd.get_model("(pk1)")
      self.atom = model.atom.pop()
      if not self.labeling:
         cmd.label("(pk1)", '""')
      elif self.atom.name == 'CA':
         cmd.label("(pk1)", '" %s %s" % (resn,resi)')
      else:
         cmd.label("(pk1)", '" %s %s" % (name,resi)')
      cmd.unpick()
      cmd.refresh_wizard()





#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------

# pmgui_tk 
# TKinter based gui for PyMol
# NOTE: must have treads support compiled into python to use this module
#
# **This is the only module which should be/need be imported by 
# **PyMol Programs

import threading
import sys
from wxPython.wx import *

ID_ABOUT = 101
ID_EXIT  = 102

class MyFrame(wxFrame):
   def __init__(self, parent, ID, title):
      wxFrame.__init__(self, parent, ID, title,
                       wxDefaultPosition, wxSize(200, 150))
      self.CreateStatusBar()
      self.SetStatusText("This is the statusbar")

      menu = wxMenu()
      menu.Append(ID_ABOUT, "&About",
                  "More information about this program")
      menu.AppendSeparator()
      menu.Append(ID_EXIT, "E&xit", "Terminate the program")

      menuBar = wxMenuBar()
      menuBar.Append(menu, "&File");

      self.SetMenuBar(menuBar)


class MyApp(wxApp):
   def OnInit(self):
      frame = MyFrame(NULL, -1, "Hello from wxPython")
      frame.Show(true)
      self.SetTopWindow(frame)
      return true

def run():
   app = MyApp(0)
   app.MainLoop()

t = threading.Thread(target=run,args=())
t.setDaemon(1)
t.start()









import chempy
import copy
from cpv import *
import operator

class Base:

#------------------------------------------------------------------------------
   def update_index(self):
      if chempy.feedback['verbose']:
         print " "+str(self.__class__)+": updating indexes..."
      c = 0
      self.index = {}
      idx = self.index
      for a in self.atom:
         idx[id(a)] = c
         c = c + 1
         
#------------------------------------------------------------------------------
   def get_residues(self):
      list = []
      if self.nAtom:
         last = self.atom[0]
         c = 0
         start = 0
         for a in self.atom:
            if not a.in_same_residue(last):
               list.append((start,c))
               start = c
               last = a
            c = c + 1
         if (c-start>1):
            list.append((start,c))
      return list

#------------------------------------------------------------------------------

   def get_coord_list(self):
      lst = []
      for a in self.atom:
         lst.append(a.coord)
      return lst

#------------------------------------------------------------------------------
   def get_mass(self):
      sm = 0.0
      for a in self.atom:
         sm = sm + a.get_mass()
      return sm

#------------------------------------------------------------------------------
   def get_nuclear_charges(self):
      '''Return the sum of nuclear charges of all atoms in a molecule.'''
      sm = 0
      for a in self.atom:
         sm = sm + a.get_number()
      return sm

#------------------------------------------------------------------------------
   def list(self):
      for a in self.atom:
         print a.symbol, a.name,  a.coord
      for a in self.bond:
         print a.index
         
#------------------------------------------------------------------------------
   def get_implicit_mass(self):
      # mass calculation for implicit models

      valence = [0]*len(self.atom)
      implicit = [0]*len(self.atom)
      
      for a in self.bond:
         ai0 = a.index[0]
         ai1 = a.index[1]
         valence[ai0] = valence[ai0] + a.order
         valence[ai1] = valence[ai1] + a.order
      c = 0
      for a in self.atom:
         valence[c] = valence[c] - a.formal_charge
         implicit[c] = a.get_implicit_valence()[valence[c]]
         c = c + 1
      h_count = reduce(operator.__add__,implicit)
      hydrogen = chempy.Atom()
      hydrogen.symbol='H'
      return self.get_mass()+hydrogen.get_mass()*h_count

#------------------------------------------------------------------------------
   def assign_names(self,preserve=1):
      dct = {}
      cnt = {}
      if preserve:
         for a in self.atom:
            if a.has('name'):
               dct[a['name']] = 1
      else:
         for a in self.atom:
            if hasattr(a,'name'):
               del a.name
      for a in self.atom:
         if not a.has('name'):
            if not cnt.has_key(a.symbol):
               c = 1
            else:
               c = cnt[a.symbol]
            while 1:
               nam = a.symbol+str(c)
               c = c + 1
               if not dct.has_key(nam):
                  break
            cnt[a.symbol]=c
            a.name=nam
            dct[nam]=1
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------

class Indexed(Base):

   attr_value = {
      'nAtom' : compile('len(self.atom)','Indexed','eval'),
      'nBond' : compile('len(self.bond)','Indexed','eval'),
      }

   def __getattr__(self,attr):
      if Indexed.attr_value.has_key(attr):
         return eval(Indexed.attr_value[attr])
      else:
         raise AttributeError(attr)

#------------------------------------------------------------------------------
   def __init__(self):
      self.reset()
      
#------------------------------------------------------------------------------
   def reset(self):
      self.index = None
      self.molecule = chempy.Molecule()
      self.atom = []
      self.bond = []

#------------------------------------------------------------------------------
   def get_min_max(self):
      if len(self.atom):
         mn = copy.deepcopy(self.atom[0].coord)
         mx = copy.deepcopy(self.atom[0].coord)
         for a in self.atom:
            ac = a.coord
            if mn[0]>ac[0]: mn[0]=ac[0]
            if mn[1]>ac[1]: mn[1]=ac[1]
            if mn[2]>ac[2]: mn[2]=ac[2]
            if mx[0]<ac[0]: mx[0]=ac[0]
            if mx[1]<ac[1]: mx[1]=ac[1]
            if mx[2]<ac[2]: mx[2]=ac[2]
         return [mn,mx]
      else:
         return [[0.0,0.0,0.0],[0.0,0.0,0.0]]
      
#------------------------------------------------------------------------------
   def merge(self,other): # steals atom objects from 'other' and resets 'other'
      if chempy.feedback['actions']:
         print " "+str(self.__class__)+": merging models..."
      nAtom=self.nAtom
      self.atom.extend(other.atom)
      for b in other.bond:
         b.index[0]=b.index[0]+nAtom
         b.index[1]=b.index[1]+nAtom
         self.bond.append(b)
      other.reset()
      if self.index:
         self.update_index()

#--------------------------------------------------------------------------------
   def delete_atom(self,index):
      if chempy.feedback['atoms']:
         print " "+str(self.__class__)+": deleting atom %d." % index
   
      nAtom=self.nAtom

# update index if it exists
      if self.index:
         idx = self.index
         for k in idx.keys():
            if idx[k] > index:
               idx[k] = idx[k] - 1
         del idx[id(self.atom[index])]

# delete atom
      del self.atom[index]

# delete bonds associated with this atom

      nBond = len(self.bond)
      templist = []
      for i in range(nBond):
         if index in self.bond[i].index:
            templist.append(i)
      for i in range(len(templist)):
         j = templist[i] - i
         del self.bond[j]

# re-index bond table
      for b in self.bond:
         if b.index[0] > index: 
            b.index[0] = b.index[0] - 1
         if b.index[1] > index: 
            b.index[1] = b.index[1] - 1

#--------------------------------------------------------------------------------
   def delete_list(self,list): # delete a list of indexed atoms

      if chempy.feedback['atoms']:
         print " "+str(self.__class__)+": deleting atoms %s." % str(list)

      nAtom=self.nAtom

      lrev = copy.deepcopy(list)
      lrev.sort()
      lrev.reverse()
      
      # generate cross-reference tables
   
      o2n = {} # old to new
      if len(lrev):
         nxt = lrev.pop()
      else:
         nxt = None
      shft = 0
      for i in range(nAtom):
         if i == nxt:
            o2n[i]=-1
            if len(lrev):
               nxt = lrev.pop()
            else:
               nxt = None
            shft = shft - 1
         else:
            ixs = i + shft
            o2n[i] = ixs

      if shft:

         # delete atoms

         new_atom = []
         for i in range(nAtom):
            if o2n[i]>=0:
               new_atom.append(self.atom[i])
         self.atom = new_atom
         
         # delete bonds 

         new_bond = []   
         for b in self.bond:
            b0 = b.index[0]
            b1 = b.index[1]
            if (o2n[b0]>=0) and (o2n[b1]>=0):
               b.index[0] = o2n[b0]
               b.index[1] = o2n[b1]
               new_bond.append(b)
         self.bond = new_bond

         # update index if it exists
         if self.index:
            self.index = {}
            idx = self.index
            i = 0
            for a in self.atom:
               idx[id(a)] = i
               i = i + 1

#------------------------------------------------------------------------------
   def insert_atom(self,index,atom):
      if chempy.feedback['atoms']:
         print " "+str(self.__class__)+': inserting atom "%s" before %d.' % (
            atom.name,index)

      nAtom=self.nAtom
      self.atom.insert(index,atom)
      
# re-index bond table
      for b in self.bond:
         if b.index[0] >= index: 
            b.index[0] = b.index[0] + 1
         if b.index[1] >= index: 
            b.index[1] = b.index[1] + 1

# update index if it exists
      if self.index:
         idx = self.index
         for k in idx.keys():
            if idx[k] >= index:
               idx[k] = idx[k] + 1
         idx[id(atom)] = index
         
#------------------------------------------------------------------------------
   def index_atom(self,atom):
      c = 0
      id_at = id(atom)
      for a in self.atom:
         if id(a)==id_at:
            return c
         c = c + 1
      return -1
      
#------------------------------------------------------------------------------
   def add_atom(self,atom):
      if chempy.feedback['atoms']:
         print " "+str(self.__class__)+': adding atom "%s".' % atom.name
      self.atom.append(atom)
      index = self.nAtom - 1
      if self.index:
         self.index[id(atom)] = index
      return index

#------------------------------------------------------------------------------
   def add_bond(self,bond):
      if chempy.feedback['bonds']:
         print " "+str(self.__class__)+": adding bond (%d,%d)." % \
               (bond.index[0],bond.index[1])
      self.bond.append(bond)      

#------------------------------------------------------------------------------
   def remove_bond(self,index):
      if chempy.feedback['bonds']:
         print " "+str(self.__class__)+": removing bond %d." % index
      nBond=len(self.Bond)
      del self.bond[index]
      
   
#------------------------------------------------------------------------------
   def convert_to_connected(self):
      if chempy.feedback['verbose']:
         print " "+str(self.__class__)+": converting to connected model..."
      model = Connected()
      model.molecule = self.molecule
      model.atom = self.atom
      model.bond = []
      model.index = None
      for a in model.atom:
         model.bond.append([])
      for b in self.bond:
         model.bond[b.index[0]].append(b) # note two refs to same object
         model.bond[b.index[1]].append(b) # note two refs to same object 
      self.reset()
      return model
#------------------------------------------------------------------------------
   def from_molobj(self,molobj): 
      self.reset()
      mol = self.molecule
      if len(molobj.title):
         mol.title = molobj.title
      if len(molobj.comments):
         mol.comments = molobj.comments
      mol.chiral = molobj.chiral
      mol.dim_code = molobj.dimcode
      for a in molobj.atom:
         at = chempy.Atom()
         at.symbol = a.symbol
         at.name = a.name
         if a.resn != chempy.Atom.defaults['resn']:
            at.resn = a.resn
         if a.resn_code != chempy.Atom.defaults['resn_code']:
            at.resn_code = a.resn_code
         at.resi = a.resi
         at.resi_number = a.resi_number
         at.b = a.b
         at.q = a.q
         at.alt = a.alt
         at.hetatm = a.hetatm
         if a.segi != chempy.Atom.defaults['segi']:
            at.segi = a.segi
         if a.chain != chempy.Atom.defaults['chain']:
            at.chain = a.chain
         at.color_code = a.color_code
         at.coord = a.coord
         at.formal_charge = a.formal_charge
         at.partial_charge = a.partial_charge
         if a.numeric_type != -99:
            at.numeric_type = a.numeric_type
         if a.text_type != 'UNKNOWN':
            at.text_type = a.text_type
         at.stereo = a.stereo
         if hasattr(a,'flags'):
            at.flags = a.flags
         if hasattr(a,'vdw'):
            at.vdw = a.vdw
         self.atom.append(at)
      for b in molobj.bond:
         bnd = chempy.Bond()
         bnd.index = [b.atom[0],b.atom[1]]
         bnd.order = b.order
         bnd.stereo = b.stereo
         self.bond.append(bnd)
#------------------------------------------------------------------------------
   def sort(self):
      if chempy.feedback['verbose']:
         print " "+__name__+": sorting..."
      if not self.index:
         self.update_index()
      old_index = self.index
      self.atom.sort()      
      self.update_index()
      xref = {}
      new_index = self.index
      for a in new_index.keys():
         xref[old_index[a]] = new_index[a]
      for b in self.bond:
         b.index[0] = xref[b.index[0]]
         b.index[1] = xref[b.index[1]]
      del old_index
      del xref

#------------------------------------------------------------------------------
   def get_internal_tuples(self):
      # generates raw atom sets needed to construct an internal coordinate
      # description of the molecule
      model = self
      # get a connected version too
      cmodel = copy.deepcopy(model).convert_to_connected()
      center = [0.0,0.0,0.0]
      nAtom = model.nAtom
      to_go = nAtom
      done = {}
      if to_go<3:
         z_set = [(0),(1,0)]
      else:
         # get center of molecule
         for a in model.atom:
            center = add(center,a.coord)
         center = scale(center,1.0/nAtom)
         # find most central multivalent atom
         min_a = -1
         c = 0
         for a in model.atom:
            if len(cmodel.bond[c])>1: # must have at least two neighbors
               d = distance(a.coord,center)
               if min_a < 0:
                  min_d = d
                  min_a = c
               elif d<min_d:
                  min_d=d
                  min_a=c
            c = c + 1
         fst = min_a
         # make this our first atom
         z_set = [( fst, )]
         done[fst] = 1
         to_go = to_go - 1
         # for the second atom, try to choose different multivalent neighbor
         nxt = -1
         for b in cmodel.bond[fst]:
            nbr = b.index[0]
            if nbr == fst:
               nbr = b.index[1]
            if len(cmodel.bond[nbr])>1:
               nxt = nbr
               break
         # safety, choose any neighbor
         if nxt<0:
            nbr = b.index[0]
            if nbr == fst:
               nbr = b.index[1]
            nxt = nbr
         z_set.append((nxt,fst))
         done[nxt] = 1
         to_go = to_go - 1
         # for the third atom, choose a different multivalent neighbor
         trd = -1
         for b in cmodel.bond[fst]:
            nbr = b.index[0]
            if nbr == fst:
               nbr = b.index[1]
            if len(cmodel.bond[nbr])>1:
               if not done.has_key(nbr):
                  trd = nbr
                  break
         # safety, choose any unchosen neighbor
         if trd<0:
            for b in cmodel.bond[fst]:
               nbr = b.index[0]
               if nbr == fst:
                  nbr = b.index[1]
               if not done.has_key(nbr):
                  trd = nbr
                  break
         z_set.append((trd,fst,nxt))
         done[trd] = 1
         result = 1
         to_go = to_go - 1
         if to_go:
            # now find all torsions in the molecule
            tors = {}
            for b in model.bond: # use bond as center of torsion
               a1 = b.index[0]
               a2 = b.index[1]
               for c in cmodel.bond[a1]: 
                  a0 = c.index[0] 
                  if a0 not in (a1,a2): # outside atom
                     for d in cmodel.bond[a2]:
                        a3 = d.index[0] 
                        if a3 not in (a0,a1,a2): # outside atom
                           if a0 < a3:
                              to = (a0,a1,a2,a3)
                           else:
                              to = (a3,a2,a1,a0)                        
                           tors[to] = 1
                        a3 = d.index[1] 
                        if a3 not in (a0,a1,a2): # outside atom
                           if a0 < a3:
                              to = (a0,a1,a2,a3)
                           else:
                              to = (a3,a2,a1,a0)
                           tors[to] = 1
                  a0 = c.index[1] 
                  if a0 not in (a1,a2): # outside atom
                     for d in cmodel.bond[a2]:
                        a3 = d.index[0] 
                        if a3 not in (a0,a1,a2): # outside atom
                           if a0 < a3:
                              to = (a0,a1,a2,a3)
                           else:
                              to = (a3,a2,a1,a0)                        
                           tors[to] = 1
                        a3 = d.index[1] 
                        if a3 not in (a0,a1,a2): # outside atom
                           if a0 < a3:
                              to = (a0,a1,a2,a3)
                           else:
                              to = (a3,a2,a1,a0)                        
                           tors[to] = 1
            if len(tors.keys()):
               # choose remaining atoms based on existing atoms using torsion
               while to_go:
                  for tor in tors.keys():
                     a0 = tor[0]
                     a1 = tor[1]
                     a2 = tor[2]
                     a3 = tor[3]
                     dh0 = done.has_key(a0)
                     dh1 = done.has_key(a1)
                     dh2 = done.has_key(a2)
                     dh3 = done.has_key(a3)
                     if ( (not dh0) and dh1 and dh2 and dh3 ):
                        z_set.append((a0,a1,a2,a3))
                        done[a0] = 1
                        to_go = to_go - 1
                     elif ( dh0 and dh1 and dh2 and (not dh3) ):
                        z_set.append((a3,a2,a1,a0))
                        done[a3] = 1
                        to_go = to_go - 1
            else: # for molecules with no torsions (dichloromethane, etc.)
               # we have to generate torsions which include one virtual
               # bond
               for b in cmodel.bond[fst]:
                  nbr = b.index[0]
                  if nbr in [fst,nxt,trd]:
                     nbr = b.index[1]
                  if not done.has_key(nbr):
                     z_set.append((nbr,trd,fst,nxt))
                     to_go = to_go - 1
                     done[nbr] = 1
      return z_set
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
class Connected(Base):

   attr_value = {
      'nAtom' : compile('len(self.atom)','Connected','eval'),
      }

   def __getattr__(self,attr):
      if Connected.attr_value.has_key(attr):
         return eval(Connected.attr_value[attr])
      else:
         raise AttributeError(attr)
      
#------------------------------------------------------------------------------
   def __init__(self):
      self.reset()
  
#------------------------------------------------------------------------------
   def reset(self):
      self.index = None
      self.molecule = chempy.Molecule()
      self.atom = []
      self.bond = []
      
#------------------------------------------------------------------------------
   def convert_to_indexed(self):
      if chempy.feedback['verbose']:
         print " "+str(self.__class__)+": converting to indexed model..."
      indexed = Indexed()
      indexed.atom = self.atom
      indexed.molecule = self.molecule
      c = 0 
      for a in self.bond:
         for b in a:
            if b.index[0] == c:
               indexed.bond.append(b)
         c = c + 1
      self.reset()
      return indexed

#------------------------------------------------------------------------------
   def insert_atom(self,index,atom):
      if chempy.feedback['atoms']:
         print " "+str(self.__class__)+': inserting atom "%s" before %d.' % (
            atom.name,index)

      nAtom=self.nAtom
      self.atom.insert(index,atom)
      
# re-index bond table
      for a in self.bonds:
         for b in a:
            if b.index[0] >= index:
               b.index[0] = b.index[0] + 1
            if b.index[1] >= index:
               b.index[1] = b.index[1] + 1

# update index if it exists
      if self.index:
         idx = self.index
         for k in idx.keys():
            if idx[k] >= index:
               idx[k] = idx[k] + 1
         idx[id(atom)] = index

#------------------------------------------------------------------------------
   def delete_atom(self,index):
      if chempy.feedback['atoms']:
         print " "+str(self.__class__)+": deleting atom %d." % index

      nAtom=self.nAtom

# update index if it exists
      if self.index:
         idx = self.index
         for k in idx.keys():
            if idx[k] > index:
               idx[k] = idx[k] - 1
         del idx[id(self.atom[index])]

# delete atom
      del self.atom[index]

# delete bonds associated with this atom

      nBond = len(self.bond)

      for a in self.bond:
         i = 0
         templist = []
         for b in a:
            if index in b.index:
               templist.append(i)
            i = i + 1
         for i in range(len(templist)):
            j = templist[i] - i
            del a[j]

# re-index bond table
      for b in self.bond:
         if b.index[0] > index: 
            b.index[0] = b.index[0] - 1
         if b.index[1] > index: 
            b.index[1] = b.index[1] - 1

#------------------------------------------------------------------------------
   def add_atom(self,atom):
      if chempy.feedback['atoms']:
         print " "+str(self.__class__)+': adding atom "%s".' % atom.name
      self.atom.append(atom)
      self.bond.append([])
      index = self.nAtom - 1
      if self.index:
         self.index[id(atom)] = index
      return index

#------------------------------------------------------------------------------
   def sort(self):
      if chempy.feedback['verbose']:
         print " "+__name__+": sorting..."
      if not self.index:
         self.update_index()
      old_index = self.index
      self.atom.sort()      
      self.update_index()
      xref = {}
      new_index = self.index
      for a in new_index.keys():
         xref[old_index[a]] = new_index[a]
      new_bond = [None] * self.nAtom
      c = 0
      tmp_list = []
      for a in self.bond:
         for b in a:
            if c==b.index[0]:
               tmp_list.append(b)
         new_bond[xref[c]] = a
         c = c + 1
      for b in tmp_list:
         b.index[0] = xref[b.index[0]]
         b.index[1] = xref[b.index[1]]
      del self.bond
      self.bond = new_bond
      del old_index
      del xref
      






import re
import string
import traceback

sele_re = re.compile(r'^[^\(].*[ \(\)\!\&\|]') # unwrapped selection?
pat_re = re.compile(r'[^ \(\)\!\&\|]*\/[^ \(\)\!\&\|]*')
num_re = re.compile(r'1|2|3|4|5|6|7|8|9|0')

def process(sele): # expand slash notation into a standard atom selection

   sele = str(sele)
   if string.find(sele,'/')<0:
      return sele
   while 1:
      mo = pat_re.search(sele)
      if mo == None:
         break
      bef = sele[:mo.start()]
      aft = sele[mo.end():]
      mat = mo.group()
      arg = string.split(mat,"/")
      la = len(arg)
      # wildcard -> blank
      arg = map(string.strip,arg)
      for c in range(len(arg)):
         if arg[c]=='*': arg[c]=''
      # defaults
      model = ''
      segment = ''
      chain = ''
      residue = ''
      name = ''
      alt = ''
      # interpret
      if mat[0]=="/": # preceeding slash, interpret left to right
         if la>1: model = arg[1]
         if la>2: segment = arg[2]
         if la>3: chain = arg[3]
         if la>4: residue = arg[4]
         if la>5: name = arg[5]
      else: # no preceeding slash, interpret right to left
         arg.reverse()
         if la>0: name = arg[0]
         if la>1: residue = arg[1]
         if la>2: chain = arg[2]
         if la>3: segment = arg[3]
         if la>4: model = arg[4]
      # lst
      lst = []
      if model!='': lst.append("("+string.replace(model,"+","|")+")")
      if segment!='': lst.append("s;"+string.replace(segment,'+',','))
      if chain!='': lst.append("c;"+string.replace(chain,'+',','))
      if residue!='':
         if num_re.search(residue)==None:
            lst.append("r;"+string.replace(residue,'+',','))
         else:
            residue = string.replace(residue,'+',',')
            residue = string.replace(residue,'-',':')
            if ((string.find(residue,',')>=0) and # compound residue specification
                (string.find(residue,':')>=0)):
               new_list = []
               for a in string.split(residue,','): # spread it out...
                  new_list.append("i;"+a)
               residue = "("+string.join(new_list,'|')+")"
               lst.append(residue)
            else:
               lst.append("i;"+residue)                        
      if name!='':
         if(string.find(name,'`')>=0): # alternate conformations present
            (name,alt) = string.split(name,'`')
            if not len(alt):
               alt="''"
            if len(name):
               lst.append("n;"+string.replace(name,'+',','))
         else:
            lst.append("n;"+string.replace(name,'+',','))
      if alt!='': lst.append("alt "+string.replace(alt,'+',','))      
      if not len(lst):
         st = "*"
      else:
         st = reduce(lambda x,y:x+"&"+y,lst)
      sele = bef+"("+st+")"+aft
   return sele




#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-*
#-* NOTE: Based on code by John E. Grayson which was in turn 
#-* based on code written by Doug Hellmann. 
#Z* -------------------------------------------------------------------

from Tkinter import *
import Pmw
import sys, string

class AbstractApp(Pmw.MegaWidget):        
   appversion      = ''
   appname         = ''
   copyright       = ''
   contactweb      = ''
   contactemail    = ''
          
   frameWidth       = 800
   frameHeight     = 120
   padx         = 1
   pady         = 1
   balloonhelp    = 1
   
   busyCursor = 'watch'
   
   def __init__(self, **kw):
      optiondefs = (
         ('padx',         1,               Pmw.INITOPT),
         ('pady',         1,               Pmw.INITOPT))
      self.defineoptions(kw, optiondefs)

      self.root = Tk()
      self.initializeTk(self.root)
      Pmw.initialise(self.root)
      self.root.title(self.appname)
      self.root.geometry('%dx%d+0+0' % (self.frameWidth, self.frameHeight))

      # Initialize the base class
      Pmw.MegaWidget.__init__(self, parent=self.root)
      
      # initialize the application
      self.appInit()
      
      # create the interface
      self.__createInterface()
      
      # create a table to hold the cursors for
      # widgets which get changed when we go busy
      self.preBusyCursors = None
      
      # pack the container and set focus
      # to ourselves
      self._hull.pack(side=LEFT, fill=BOTH, expand=YES)
      self.focus_set()

      # initialize our options
      self.initialiseoptions(AbstractApp)
      
   def appInit(self):
      # Called before interface is created (should be overridden).
      pass
      
   def initializeTk(self, root):
      self.pad = ' ' 
      # Initialize platform-specific options
      if sys.platform == 'mac':
         self.__initializeTk_mac(root)
      elif sys.platform[:3] == 'win':
         self.__initializeTk_win32(root)
      elif sys.platform[:5] == 'linux':
         self.frameHeight=self.frameHeight+24
         self.__initializeTk_unix(root)
      else:
         self.__initializeTk_unix(root)

   def __initializeTk_colors_common(self, root):
      root.option_add('*background', 'grey')
      root.option_add('*foreground', 'black')
      root.option_add('*EntryField.Entry.background', 'white')
      root.option_add('*Entry.background', 'white')      
      root.option_add('*MessageBar.Entry.background', 'gray85')
      root.option_add('*Listbox*background', 'white')
      root.option_add('*Listbox*selectBackground', 'dark slate blue')
      root.option_add('*Listbox*selectForeground', 'white')
                  
   def __initializeTk_win32(self, root):
      self.__initializeTk_colors_common(root)
      root.option_add('*Font', 'Tahoma 8')
      self.pad = ' '
#      root.option_add('*EntryField.Entry.Font', 'Courier 10')
#      root.option_add('*Listbox*Font', 'Courier 10')
      
   def __initializeTk_mac(self, root):
      self.__initializeTk_colors_common(root)
      
   def __initializeTk_unix(self, root):
      self.__initializeTk_colors_common(root)

   def busyStart(self, newcursor=None):
      if not newcursor:
         newcursor = self.busyCursor
      newPreBusyCursors = {}
      for component in self.busyWidgets:
         newPreBusyCursors[component] = component['cursor']
         component.configure(cursor=newcursor)
         component.update_idletasks()
      self.preBusyCursors = (newPreBusyCursors, self.preBusyCursors)
      
   def busyEnd(self):
      if not self.preBusyCursors:
         return
      oldPreBusyCursors = self.preBusyCursors[0]
      self.preBusyCursors = self.preBusyCursors[1]
      for component in self.busyWidgets:
         try:
            component.configure(cursor=oldPreBusyCursors[component])
         except KeyError:
            pass
         component.update_idletasks()
           
   def __createAboutBox(self):
      Pmw.aboutversion(self.appversion)
      Pmw.aboutcopyright(self.copyright)
      Pmw.aboutcontact(
        'For more information, browse to: %s\n or send email to: %s' %\
                 (self.contactweb, self.contactemail))
      self.about = Pmw.AboutDialog(self._hull, 
                            applicationname=self.appname)
      self.about.withdraw()
      return None
   
   def showAbout(self):
      # Create the dialog to display about and contact information.
      self.about.activate(geometry='centerscreenfirst')
#      self.about.show()
#      self.about.focus_set()
      
   def toggleBalloon(self):
      if self.toggleBalloonVar.get():
         self.__balloon.configure(state = 'both')
      else:
         self.__balloon.configure(state = 'status')

   def __createMenuBar(self):
      self.menuBar = self.createcomponent('menubar', (), None,
                                 Pmw.MenuBar,
                                 (self._hull,),
#                                 hull_relief=RAISED,
#                                 hull_borderwidth=0,
                                 balloon=self.balloon())

      self.menuBar.pack(fill=X)
      self.menuBar.addmenu('Help', 'About %s' % self.appname, side='right')
      self.menuBar.addmenu('File', 'File Input')
      self.menuBar.addmenu('Edit', 'Text Editing')
                     
   def createMenuBar(self):
#override this
      pass

   def __createBalloon(self):
      # Create the balloon help manager for the frame.
      # Create the manager for the balloon help
      self.__balloon = self.createcomponent('balloon', (), None,
                                   Pmw.Balloon, (self._hull,))

   def balloon(self):
      return self.__balloon

   def __createDataArea(self):
      # Create data area where data entry widgets are placed.
      self.dataArea = self.createcomponent('dataarea',
                                  (), None,
                                  Frame, (self._hull,), 
                                  relief=SUNKEN, 
                                  bd=1)
      self.dataArea.pack(side=LEFT, fill=BOTH, expand=YES,
                     padx=1, pady=1)

   def __createCommandArea(self):
      # Create a command area for application-wide buttons.
      self.commandFrame = self.createcomponent('commandframe', (), None,
         Frame,(self._hull,),relief=SUNKEN,bd=1)
      self.commandFrame.place(width=500)
      self.commandFrame.pack(side=TOP, 
                   expand=NO, 
                   fill=BOTH,
                   padx=1,
                   pady=1)

   def __createMessageBar(self):
      # Create the message bar area for help and status messages.
      frame = self.createcomponent('bottomtray', (), None,
                            Frame,(self._hull,), relief=SUNKEN)
      self.__messageBar = self.createcomponent('messagebar',
                                      (), None,
                                     Pmw.MessageBar, 
                                     (frame,),
                                     #entry_width = 40,
                                     entry_relief=SUNKEN,
                                     entry_bd=1,
                                     labelpos=None)
      self.__messageBar.pack(side=LEFT, expand=YES, fill=X)

#      self.__progressBar = ProgressBar.ProgressBar(frame,
#                                    fillColor='slateblue',
#                                    doLabel=1,
#                                    width=150)
#      self.__progressBar.frame.pack(side=LEFT, expand=NO, fill=NONE)

#      self.updateProgress(0)
      frame.pack(side=BOTTOM, expand=NO, fill=X)
               
      self.__balloon.configure(statuscommand = \
                         self.__messageBar.helpmessage)

   def messageBar(self):
      return self.__messageBar

#   def updateProgress(self, newValue=0, newMax=0):
#      self.__progressBar.updateProgress(newValue, newMax)

   def bind(self, child, balloonHelpMsg, statusHelpMsg=None):
      # Bind a help message and/or status message to a widget.
      self.__balloon.bind(child, balloonHelpMsg, statusHelpMsg)

   def get_dataArea(self):
      # Retrieve the interior site where widgets should go.
      return self.dataArea

   def get_commandFrame(self):
      # Retrieve the command frame where buttons go.
      return self.commandFrame

   def __createInterface(self):
      self.__createBalloon()
      self.__createMenuBar()
      self.__createDataArea()
      self.__createCommandArea()
      self.__createMessageBar()
      self.__createAboutBox()
      #
      # Create the parts of the interface
      # which can be modified by subclasses
      #
      self.busyWidgets = ( self.root, )
      self.createMenuBar()
      self.createInterface()

   def createInterface(self):
      # Override this method to create the interface for the app.
      pass
      
   def quit_app(self):
      # Override this method to create the interface for the app.
      pass
      
   def main(self):
      # This method should be left intact!
      self.pack()
      self.mainloop()
      self.quit_app()
      
   def run(self):
      self.main()





import os

def machine_get_clipboard():
   result = []
   from popen2 import popen2
   wish_path = os.environ['PYMOL_PATH']+"/ext/bin/wish8.0"
   if not os.path.exists(wish_path):
      wish_path = "/usr/bin/wish"
   if os.path.exists(wish_path):
      pipe = popen2(wish_path)
      pipe[1].write("puts [ selection get ]\n")
      pipe[1].write("exit\n")
      pipe[1].close()
      while 1:
         l=pipe[0].readline()
         if not l: break
         result.append(l)
   return result

                 



#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-*
#-* NOTE: Based on code by John E. Grayson which was in turn 
#-* based on code written by Doug Hellmann. 
#Z* -------------------------------------------------------------------

from Tkinter import *
from tkFileDialog import *

from AbstractApp import AbstractApp
from Setting import Setting
from SetEditor import SetEditor
from ColorEditor import ColorEditor
from Demo import Demo

import Pmw
import sys, string
import pymol
from pymol import cmd
from pymol import util
from pymol import parser

import re
import thread
import threading
import os
from glob import glob
import __builtin__
import traceback
import Queue

def complete(event,str,widget,self):
   st = parser.complete(str.get())
   if st:
      str.set(st)
      widget.icursor(len(st))
   self.focus_entry = 1
   return 1

class PMGApp(AbstractApp):

   appversion     = '0.82'
   appname       = 'PyMOL Molecular Graphics System'
   copyright      = 'Copyright (C) 1998-2002 by Warren DeLano of\nDeLano Scientific. All rights reserved.'
   contactweb     = 'http://www.pymol.org'
   contactemail   = 'warren@delanoscientific.com'

   def appInit(self): # create a global variable for the external gui
      pymol._ext_gui = self
      self.fifo = Queue.Queue(0)

   def execute(self,cmmd): 
      self.fifo.put(cmmd)

   def buttonAdd(self,frame,text,cmd):
      newBtn=self.createcomponent('button', (), None,
         Button,frame,text=text,highlightthickness=0,
         command=cmd,padx=0,pady=0)
      newBtn.pack(side=LEFT,fill=BOTH,expand=YES)
      
   def createButtons(self):
      
      row2 = self.createcomponent('row2', (), None,
         Frame,self.get_commandFrame(),bd=0)
      row2.pack(side=TOP,fill=BOTH,expand=YES)
      btn_reset = self.buttonAdd(row2,'Reset',lambda: cmd.do("_ reset"))
      btn_reset = self.buttonAdd(row2,'Zoom',lambda: cmd.do("_ zoom"))      
      btn_rtrace = self.buttonAdd(row2,'Ray',lambda : cmd.do("_ ray"))
      btn_reset = self.buttonAdd(row2,'Rock',lambda :cmd.do("_ rock"))

      row3 = self.createcomponent('row3', (), None,
         Frame,self.get_commandFrame(),bd=0)
      row3.pack(side=TOP,fill=BOTH,expand=YES)
      btn_unpick = self.buttonAdd(row3,'Unpick',lambda : cmd.do("_ unpick"))
      btn_hidesele = self.buttonAdd(row3,'Hide Sele',self.hide_sele)
      btn_getview = self.buttonAdd(row3,'Get View',lambda : cmd.get_view()) # doesn't get logged

#      row3 = self.createcomponent('row3', (), None,
#         Frame,self.get_commandFrame(),bd=0)
#      row3.pack(side=TOP,fill=BOTH,expand=YES)
#      btn_reset = self.buttonAdd(row3,'Barf',self.quit)
#      btn_reset = self.buttonAdd(row3,'Now',self.quit)
#      btn_reset = self.buttonAdd(row3,'Eat',self.quit)
#      btn_reset = self.buttonAdd(row3,'Shrimp',self.quit)

      row1 = self.createcomponent('row1', (), None,
         Frame,self.get_commandFrame(),bd=0)
      row1.pack(side=TOP,fill=BOTH,expand=YES)
      btn_rewind = self.buttonAdd(row1,'|<',lambda : cmd.do("_ rewind"))
      btn_back = self.buttonAdd(row1,'<',lambda : cmd.do("_ backward"))
      btn_stop = self.buttonAdd(row1,'Stop',lambda : cmd.do("_ mstop"))
      btn_play = self.buttonAdd(row1,'Play',lambda : cmd.do("_ mplay"))
      btn_forward = self.buttonAdd(row1,'>',lambda : cmd.do("_ forward"))
      btn_last = self.buttonAdd(row1,'>|',lambda : cmd.do("_ ending"))
      btn_ccache = self.buttonAdd(row1,'MClear',lambda : cmd.do("_ mclear"))

   def my_show(self,win,center=1):
      if sys.platform!='linux2':
         win.show()
      else: # autocenter, deiconify, and run mainloop
         # this is a workaround for a bug in the
         # interaction between Tcl/Tk and common Linux
         # window managers (namely KDE/Gnome) which causes
         # an annoying 1-2 second delay in opening windows!
         if center:
            tw = win.winfo_reqwidth()+100
            th = win.winfo_reqheight()+100
            vw = win.winfo_vrootwidth()
            vh = win.winfo_vrootheight()
            x = max(0,(vw-tw)/2)
            y = max(0,(vh-tw)/2)
            win.geometry(newGeometry="+%d+%d"%(x,y))
         win.deiconify()
#         win.show()
         
   def my_withdraw(self,win):
      if sys.platform!='linux2':
         win.withdraw()
      else: # autocenter, deiconify, and run mainloop
         win.destroy()

   def my_activate(self,win,center=1,focus=None):
      if sys.platform!='linux2':
         win.activate()
      else: # autocenter, deiconify, and run mainloop
         # this is a workaround for a bug in the
         # interaction between Tcl/Tk and common Linux
         # window managers (namely KDE/Gnome) which causes
         # an annoying 1-2 second delay in opening windows!
         if center:
            tw = win.winfo_reqwidth()+100
            th = win.winfo_reqheight()+100
            vw = win.winfo_vrootwidth()
            vh = win.winfo_vrootheight()
            x = max(0,(vw-tw)/2)
            y = max(0,(vh-tw)/2)
            win.geometry(newGeometry="+%d+%d"%(x,y))
         win.deiconify()
         if focus!=None:
            focus.focus_set()
         win.mainloop()
         
   def my_deactivate(self,win):
      if sys.platform!='linux2':
         win.deactivate()
      else: # autocenter, deiconify, and run mainloop
         win.destroy()

   def createMain(self):
      self.command = StringVar()
      self.entry = self.createcomponent('entry', (), None,
                           Entry,
                           (self.get_dataArea(),),
                           justify=LEFT,
                           width=50,
                           textvariable=self.command)
      self.entry.pack(side=BOTTOM,expand=NO,fill=X)
      self.entry.bind('<Return>',lambda event,w=self.command:
         (cmd.do(w.get()),cmd.dirty(),w.set('')))

      self.entry.bind('<Tab>',lambda event,w=self.command,
                      e=self.entry,s=self:
                      complete(event,w,e,s))

      self.output = self.createcomponent('output', (), None,
                           Pmw.ScrolledText,
                           (self.get_dataArea(),))

      text = self.output.component('text')
      if sys.platform[:5]=='linux':
         self.my_fw_font=('lucida console',12)
      elif sys.platform[:3]=='win':
         self.my_fw_font=('lucida console',8) # Courier 9
      else:
         self.my_fw_font=('lucida console',10)
                                                                               
      text.configure(font = self.my_fw_font)
      text.configure(width=72)
      self.focus_entry=0
      self.output.after(1000,self.flush_commands)
      self.output.after(1000,self.update_feedback)
      self.output.after(1000,self.update_menus)
      self.output.pack(side=BOTTOM,expand=YES,fill=BOTH)
      self.bind(self.entry, 'Command Input Area')
      self.initialdir = os.getcwd()
      self.log_file = "log.pml"

   def flush_commands(self):
      # flush the external GUI fifo command queue
      while not self.fifo.empty():
         try:
            cmmd = self.fifo.get(0)
            exec cmmd
         except:
            traceback.print_exc()
      self.output.after(20,self.flush_commands) # 50X a second
      
   def update_feedback(self):
      if self.focus_entry:
         self.focus_entry=0
         self.entry.focus_set()
      for a in cmd.get_feedback():
         self.output.insert(END,"\n")
         self.output.insert(END,a)
         self.output.see(END)
         self.lineCount = self.lineCount + 1
         if self.lineCount > 10000:
            self.output.delete('0.0','%i.%i' % (self.lineCount-5000,0))
            self.lineCount=5000
         self.entry.focus_set()
      self.output.after(100,self.update_feedback) # 10X a second

   def update_menus(self):
      self.setting.refresh()
      self.output.after(500,self.update_menus) # twice a second
      
   def createInterface(self):
         AbstractApp.createInterface(self)
         self.createButtons()
         self.createMain()
         self.lineCount = 0
         raw_list = glob(os.environ['PYMOL_PATH']+"/modules/pmg_tk/startup/*.py*")
         unique = {}
         for a in raw_list:
            unique[re.sub(r".*\/|\.py.*$","",a)] = 1
         for name in unique.keys():
            if name != "__init__":
               mod_name = "pmg_tk.startup."+name
               __builtin__.__import__(mod_name)
               mod = sys.modules[mod_name]
               mod.__init__(self)

   def quit_app(self):
      cmd.log_close()
      cmd.quit()  # avoid logging this - it is inconvenient...

   def file_open(self):
      ofile = askopenfilename(initialdir = self.initialdir,
                              filetypes=[("All Readable","*.pdb"),
                                         ("All Readable","*.ccp4"),
                                         ("All Readable","*.xplor"),
                                         ("All Readable","*.mol"),                                         
                                         ("All Readable","*.sdf"),
                                         ("All Readable","*.xyz"),                                         
                                         ("All Readable","*.r3d"),
                                         ("All Readable","*.cc1"),
                                         ("All Readable","*.cc2"),                                         
                                         ("All Readable","*.ent"),
                                         ("All Readable","*.dat"),
                                         ("All Readable","*.out"),
                                         ("All Readable","*.mmd"),
                                         ("All Readable","*.mmod"),
                                         ("PDB File","*.pdb"),
                                         ("All Files","*.*"),
                                         ("All Files","*"),                                         
                                         ("CCP4 Map","*.ccp4"),                                         
                                         ("PDB File","*.ent"),
                                         ("Macromodel File","*.dat"),
                                         ("Macromodel File","*.out"),
                                         ("Macromodel File","*.mmd"),
                                         ("Macromodel File","*.mmod"),
                                         ("MOL File","*.mol"),
                                         ("ChemPy Model","*.pkl"),
                                         ("Raster3D Scene","*.r3d"),
                                         ("SDF File","*.sdf"),
                                         ("XPLOR Map","*.xplor"),
                                         ("ChemDraw3D File","*.cc1"),
                                         ("ChemDraw3D File","*.cc2"),
                                         ("Tinker XYZ File","*.xyz")
                                         ])
      if len(ofile):
         self.initialdir = re.sub(r"[^\/\\]*$","",ofile)         
         cmd.log("load %s\n"%ofile,"cmd.load('%s')\n"%ofile)
         cmd.load(ofile)

   def log_open(self):
      sfile = asksaveasfilename(initialfile = self.log_file,
                                initialdir = self.initialdir,
                                filetypes=[
                                           ("PyMOL Script","*.pml"),
                                           ("PyMOL Program","*.pym"),
                                           ("Python Program","*.py"),
                                           ("All Files","*.*"),
                                           ("All Files","*"),
                                           ])
      if len(sfile):
         self.initialdir = re.sub(r"[^\/\\]*$","",sfile)
         self.log_file = re.sub(r"^.*[^\/\\]","",sfile)
         cmd.log_open(sfile)

   def log_resume(self,append_only=0):
      ofile = askopenfilename(initialdir = os.getcwd(),
                   filetypes=[("All Resumable","*.pml"),
                              ("All Resumable","*.pym"),
                              ("All Resumable","*.py"),
                              ("PyMOL Script","*.pml"),
                              ("PyMOL Program","*.pym"),
                              ("Python Program","*.py"),
                              ("All Files","*.*"),                                           
                              ("All Files","*"),
                              ])
      if len(ofile):
         self.initialdir = re.sub(r"[^\/\\]*$","",ofile)
         self.log_file = re.sub(r"^.*[^\/\\]","",ofile)
         os.chdir(self.initialdir)	         
         cmd.resume(ofile)

   def log_append(self,append_only=0):
      ofile = askopenfilename(initialdir = os.getcwd(),
                   filetypes=[("All Appendable","*.pml"),
                              ("All Appendable","*.pym"),
                              ("All Appendable","*.py"),
                              ("PyMOL Script","*.pml"),
                              ("PyMOL Program","*.pym"),
                              ("Python Program","*.py"),
                              ("All Files","*.*"),                                           
                              ("All Files","*"),
                              ])
      if len(ofile):
         self.initialdir = re.sub(r"[^\/\\]*$","",ofile)
         self.log_file = re.sub(r"^.*[^\/\\]","",ofile)
         os.chdir(self.initialdir)	         
         cmd.log_open(ofile,'a')

   def file_save(self):
      lst = cmd.get_names('all')
      lst = filter(lambda x:x[0]!="_",lst)
      self.dialog = Pmw.SelectionDialog(self.root,title="Save",
                          buttons = ('OK', 'Cancel'),
                                   defaultbutton='OK',
                          scrolledlist_labelpos=N,
                          label_text='Which object or selection would you like to save?',
                          scrolledlist_items = lst,
                          command = self.file_save2)
      if len(lst):
         listbox = self.dialog.component('scrolledlist')      
         listbox.selection_set(0)
      self.my_show(self.dialog)
      
   def file_save2(self,result):
      if result!='OK':
         self.my_withdraw(self.dialog)
         del self.dialog
      else:
         sels = self.dialog.getcurselection()
         if len(sels)!=0:
            sfile = sels[0]+".pdb"
            self.my_withdraw(self.dialog)
            del self.dialog
            if result=='OK':
               sfile = asksaveasfilename(initialfile = sfile,
                                         initialdir = self.initialdir,
                                         filetypes=[
                                                    ("PDB File","*.pdb"),
                                                    ("MOL File","*.mol"),
                                                    ("MMD File","*.mmd"),
                                                    ("PKL File","*.pkl"),
                                                    ])
               if len(sfile):
                  self.initialdir = re.sub(r"[^\/\\]*$","",sfile)
                  cmd.log("save %s,(%s)\n"%(sfile,sels[0]),
                          "cmd.save('%s','(%s)')\n"%(sfile,sels[0]))
                  cmd.save(sfile,"(%s)"%sels[0])

   def hide_sele(self):
      cmd.log("util.hide_sele()\n","util.hide_sele()\n")
      util.hide_sele()
         
   def file_run(self):
      ofile = askopenfilename(initialdir = os.getcwd(),
                   filetypes=[("All Runnable","*.pml"),
                              ("All Runnable","*.pym"),
                              ("All Runnable","*.py"),
                              ("All Runnable","*.pyc"),
                              ("PyMOL Script","*.pml"),
                              ("Python Program","*.py"),
                              ("Python Program","*.pyc"),
                              ("PyMOL Program","*.pym"),
                              ("All Files","*.*"),                                           
                              ("All Files","*"),
                              ])
      if len(ofile):
         dir = re.sub(r"[^\/\\]*$","",ofile)
         os.chdir(dir)	
         if re.search("\.pym*$|\.PYM*$",ofile):
            cmd.do("run "+ofile);      
         else:
            cmd.do("@"+ofile);

   def file_savepng(self):
      sfile = asksaveasfilename(initialdir = self.initialdir,
             filetypes=[("PNG File","*.png")])
      if len(sfile):
         self.initialdir = re.sub(r"[^\/\\]*$","",sfile)
         cmd.log("png %s\n"%sfile,"cmd.png('%s')\n"%sfile)
         cmd.png(sfile)
         
      
   def file_savemovie(self):
      sfile = asksaveasfilename(filetypes=[("Numbered PNG Files","*.png")])
      if len(sfile):
         self.initialdir = re.sub(r"[^\/\\]*$","",sfile)
         cmd.log("mpng %s\n"%sfile,"cmd.mpng('%s')\n"%sfile)         
         cmd.mpng(sfile)
   
   def createMenuBar(self):
      self.menuBar.addmenuitem('Help', 'command',
                               'Get information on application', 
                               label='About', command = lambda : cmd.do("_ splash"))

      self.menuBar.addmenuitem('Help', 'command', 'Release Notes',
                               label='Release Notes',
                               command = lambda: cmd.do("_ cmd.show_help('release')"))

      self.menuBar.addmenuitem('Help', 'separator', '')

      self.menuBar.addmenuitem('Help', 'command', 'Help on Commands',
                               label='Commands',
                               command = lambda: cmd.do("_ cmd.show_help('commands')"))

      self.menuBar.addmenuitem('Help', 'command', 'Help on Launching',
                               label='Launching',
                               command = lambda: cmd.do("_ cmd.show_help('launching')"))      

      self.menuBar.addmenuitem('Help', 'separator', '')

      self.menuBar.addmenuitem('Help', 'command', 'Help on Selections',
                               label='Select Command',
                               command = lambda: cmd.do("_ cmd.show_help('select')"))      

      self.menuBar.addmenuitem('Help', 'command', 'Help on Selections',
                               label='Selection Syntax',
                               command = lambda: cmd.do("_ cmd.show_help('selections')"))      

      self.menuBar.addmenuitem('Help', 'command', 'Example Selections',
                               label='Selection Examples',
                               command = lambda: cmd.do("_ cmd.show_help('examples')"))      

      self.menuBar.addmenuitem('Help', 'separator', '')
      

      self.menuBar.addmenuitem('Help', 'command', 'Help on the Mouse',
                               label='Mouse',
                               command = lambda: cmd.do("_ cmd.show_help('mouse')"))      

      self.menuBar.addmenuitem('Help', 'command', 'Help on the Keyboard',
                               label='Keyboard',
                               command = lambda: cmd.do("_ cmd.show_help('keyboard')"))      

      self.menuBar.addmenuitem('Help', 'command', 'Help on Molecular Editing',
                               label='Molecular Editing',
                               command = lambda: cmd.do("_ cmd.show_help('editing')"))      

      self.menuBar.addmenuitem('Help', 'command', 'Help on Molecular Editing',
                               label='Molecular Editing Keys',
                               command = lambda: cmd.do("_ cmd.show_help('edit_keys')"))      

      self.menuBar.addmenuitem('Help', 'command', 'Help on Stereo',
                               label='Stereo',
                               command = lambda: cmd.do("_ cmd.show_help('stereo')"))      

      self.menuBar.addmenuitem('Help', 'separator', '')
      

      self.menuBar.addmenuitem('Help', 'command', 'Help on the API',
                               label='API',
                               command = lambda: cmd.do("_ cmd.show_help('api')"))      

      self.toggleBalloonVar = IntVar()
      self.toggleBalloonVar.set(1)
      self.setting = Setting()

      self.menuBar.addmenuitem('Help', 'separator', '')
      
      self.menuBar.addmenuitem('Help', 'checkbutton',
                         'Toggle balloon help',
                         label='Balloon help',
                        variable = self.toggleBalloonVar,
                        command=self.toggleBalloon)

      self.menuBar.addmenuitem('File', 'command', 'Open structure file.',
                        label=self.pad+'Open...',
                        command=self.file_open)

      self.menuBar.addmenuitem('File', 'command', 'Save structure file.',
                        label=self.pad+'Save Molecule...',
                        command=self.file_save)

#      self.menuBar.addmenuitem('File', 'command', 'Open sequential files.',
#                        label=self.pad+'Open Sequence...',
#                        command=self.file_open)

      self.menuBar.addmenuitem('File', 'command', 'Save current image.',
                        label=self.pad+'Save Image...',
                        command=self.file_savepng)

      self.menuBar.addmenuitem('File', 'command', 'Save all frames.',
                        label=self.pad+'Save Movie...',
                        command=self.file_savemovie)

      self.menuBar.addmenuitem('File', 'separator', '')
      
      self.menuBar.addmenuitem('File', 'command', 'Open log file.',
                        label=self.pad+'Log...',
                        command=self.log_open)

      self.menuBar.addmenuitem('File', 'command', 'Resume log file.',
                        label=self.pad+'Resume...',
                        command=self.log_resume)

      self.menuBar.addmenuitem('File', 'command', 'Append log file.',
                        label=self.pad+'Append...',
                        command=self.log_append)

      self.menuBar.addmenuitem('File', 'command', 'Close log file.',
                        label=self.pad+'Close Log',
                        command=cmd.log_close)

      self.menuBar.addmenuitem('File', 'command', 'Run program or script.',
                        label=self.pad+'Run...',
                        command=self.file_run)


      self.menuBar.addmenuitem('File', 'separator', '')

      self.menuBar.addmenuitem('File', 'command', 'Quit PyMOL',
                        label=self.pad+'Quit',
                        command=self.quit)

      self.menuBar.addmenuitem('File', 'separator', '')
      
      self.menuBar.addmenuitem('File', 'checkbutton',
                         'Log Conformations.',
                         label=self.pad+'Log Conformations',
                        variable = self.setting.log_conformations,
                        command = lambda s=self: s.setting.update('log_conformations'))

      self.menuBar.addmenuitem('File', 'checkbutton',
                         'Log Box Selections.',
                         label=self.pad+'Log Box Selections',
                        variable = self.setting.log_box_selections,
                        command = lambda s=self: s.setting.update('log_box_selections'))

      self.menuBar.addmenuitem('Edit', 'command',
                         'To Copy: Use Ctrl-C',
                         label='To copy text use Ctrl-C',
                               state='disabled',
                        command =  None)

      self.menuBar.addmenuitem('Edit', 'command',
                         'To Paste, Use Ctrl-V',
                         label='To paste text use Ctrl-V',
                               state='disabled',                               
                        command =  None)

      self.menuBar.addmenuitem('Edit', 'separator', '')

      self.menuBar.addmenuitem('Edit', 'command', 'Undo Conformation',
                               label='Undo Conformation [Ctrl-Z]',
                               command = lambda: cmd.do("_ undo"))

      self.menuBar.addmenuitem('Edit', 'command', 'Redo Conformation',
                               label='Redo Conformation [Ctrl-A]',
                               command = lambda: cmd.do("_ redo"))

      self.menuBar.addmenuitem('Edit', 'separator', '')
      
      self.menuBar.addmenuitem('Edit', 'command', 'Cycle Bond Valence',
                               label='Cycle Bond Valence [Ctrl-W]',
                               command = lambda: cmd.do("_ cycle_valence"))

      self.menuBar.addmenuitem('Edit', 'command', 'Fill Hydrogens',
                               label='Fill Hydrogens on (pk1) [Ctrl-R]',
                               command = lambda: cmd.do("_ h_fill"))

      self.menuBar.addmenuitem('Edit', 'command', 'Invert',
                               label='Invert (lb)-(pk1)-(rb) [Ctrl-E]',
                               command = lambda: cmd.do("_ invert"))

      self.menuBar.addmenuitem('Edit', 'command', 'Form Bond',
                               label='Create Bond (lb)-(rb) [Ctrl-T]',
                               command = lambda: cmd.do("_ bond"))


      self.menuBar.addmenuitem('Edit', 'separator', '')

      
      self.menuBar.addmenuitem('Edit', 'command', 'Remove (pk1)',
                               label='Remove (pk1) [Ctrl-D]',
                               command = lambda: cmd.do("_ remove pk1"))

      self.menuBar.addmenuitem('Edit', 'command', 'Remove (pkfrag1)',
                               label='Remove (pkfrag1) [Ctrl-X]',
                               command = lambda: cmd.do("_ remove pkfrag1"))

      self.menuBar.addmenuitem('Edit', 'command', 'Remove (pkchain)',
                               label='Remove (pkchain)',
                               command = lambda: cmd.do("_ remove pkchain"))

      self.menuBar.addmenuitem('Edit', 'separator', '')
      
      self.menuBar.addmenuitem('Edit', 'command', 'Make Positive',
                               label='Make (pk1) Positive [Ctrl-K]',
                               command = lambda: cmd.do("_ alter pk1,formal_charge=1.0"))

      self.menuBar.addmenuitem('Edit', 'command', 'Make Negative',
                               label='Make (pk1) Negative [Ctrl-J]',
                               command = lambda: cmd.do("_ alter pk1,formal_charge=-1.0"))

      self.menuBar.addmenuitem('Edit', 'command', 'Make Neutral',
                               label='Make (pk1) Neutral',
                               command = lambda: cmd.do("_ alter pk1,formal_charge=-0.0"))


      self.menuBar.addmenu('Fragment', 'Fragment')

      self.menuBar.addmenuitem('Fragment', 'command', 'Acetylene',
                               label='Acetylene [Alt-J]',
                               command = lambda: cmd.do(
         "_ editor.attach_fragment('pk1','acetylene',2,0)"))

      self.menuBar.addmenuitem('Fragment', 'command', 'Amide N->C',
                               label='Amide N->C [Alt-1]',
                               command = lambda: cmd.do(
         "_ editor.attach_fragment('pk1','formamide',3,1)"))

      self.menuBar.addmenuitem('Fragment', 'command', 'Amide C->N',
                               label='Amide C->N [Alt-2]',
                               command = lambda: cmd.do(
         "_ editor.attach_fragment('pk1','formamide',5,0)"))

      self.menuBar.addmenuitem('Fragment', 'command', 'Bromine',
                               label='Bromine [Ctrl-B]',
                               command = lambda: cmd.do("_ replace Br,1,1"))

      self.menuBar.addmenuitem('Fragment', 'command', 'Carbon',
                               label='Carbon [Ctrl-C]',
                               command = lambda: cmd.do("_ replace C,4,4"))

      self.menuBar.addmenuitem('Fragment', 'command', 'Carbonyl',
                               label='Carbonyl [Alt-0]',
                               command = lambda: cmd.do(
         "_ editor.attach_fragment('pk1','formaldehyde',2,0)"))

      self.menuBar.addmenuitem('Fragment', 'command', 'Chlorine',
                               label='Chlorine [Ctrl-L]',
                               command = lambda: cmd.do("_ replace Cl,1,1"))

      self.menuBar.addmenuitem('Fragment', 'command', 'Cyclobutyl',
                               label='Cyclobutyl [Alt-4]',
                               command = lambda: cmd.do(
         "_ editor.attach_fragment('pk1','cyclobutane',4,0)"))

      self.menuBar.addmenuitem('Fragment', 'command', 'Cyclopentyl',
                               label='Cyclopentyl [Alt-5]',
                               command = lambda: cmd.do(
         "_ editor.attach_fragment('pk1','cyclopentane',5,0)"))

      self.menuBar.addmenuitem('Fragment', 'command', 'Cyclopentadiene',
                               label='Cyclopentadiene [Alt-8]',
                               command = lambda: cmd.do(
           "_ editor.attach_fragment('pk1','cyclopentadiene',5,0)"))

      self.menuBar.addmenuitem('Fragment', 'command', 'Cyclohexyl',
                               label='Cyclohexyl [Alt-6]',
                               command = lambda: cmd.do(
         "_ editor.attach_fragment('pk1','cyclohexane',7,0)"))

      self.menuBar.addmenuitem('Fragment', 'command', 'Cycloheptyl',
                               label='Cycloheptyl [Alt-7]',
                               command = lambda: cmd.do(
         "_ editor.attach_fragment('pk1','cycloheptane',8,0)"))

      self.menuBar.addmenuitem('Fragment', 'command', 'Fluorine',
                               label='Fluorine [Ctrl-F]',
                               command = lambda: cmd.do("_ replace F,1,1"))

      self.menuBar.addmenuitem('Fragment', 'command', 'Iodine',
                               label='Iodine [Ctrl-I]',
                               command = lambda: cmd.do("_ replace I,1,1"))

      self.menuBar.addmenuitem('Fragment', 'command', 'Methane',
                               label='Methane',
                               command = lambda: cmd.do(
         "_ editor.attach_fragment('pk1','methane',1,0)"))

      self.menuBar.addmenuitem('Fragment', 'command', 'Nitrogen',
                               label='Nitrogen [Ctrl-N]',
                               command = lambda: cmd.do("_ replace N,4,3"))

      self.menuBar.addmenuitem('Fragment', 'command', 'Oxygen',
                               label='Oxygen [Ctrl-O]',
                               command = lambda: cmd.do("_ replace O,4,2"))

      self.menuBar.addmenuitem('Fragment', 'command', 'Phenyl',
                               label='Phenyl [Alt-9]',
                               command = lambda: cmd.do(
         "_ editor.attach_fragment('pk1','benzene',6,0)"))

      self.menuBar.addmenuitem('Fragment', 'command', 'Sulfer',
                               label='Sulfer [Ctrl-S]',
                               command = lambda: cmd.do("_ replace S,2,2"))

      self.menuBar.addmenuitem('Fragment', 'command', 'Sulfonyl',
                               label='Sulfonyl [Alt-3]',
                               command = lambda: cmd.do(
         "_ editor.attach_fragment('pk1','sulfone',3,1)"))

      self.menuBar.addmenuitem('Fragment', 'command', 'Phosphorus',
                               label='Phosphorus [Ctrl-P]',
                               command = lambda: cmd.do("_ replace P,4,3"))

      self.menuBar.addmenu('Residue', 'Residue')

      self.menuBar.addmenuitem('Residue', 'command', 'Acetyl',
                               label='Acetyl [Alt-B]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','ace')"))

      self.menuBar.addmenuitem('Residue', 'command', 'Alanine',
                               label='Alanine [Alt-A]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','ala')"))

      self.menuBar.addmenuitem('Residue', 'command', 'Aspartate',
                               label='Aspartate [Alt-D]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','asp')"))

      self.menuBar.addmenuitem('Residue', 'command', 'Asparagine',
                               label='Asparagine [Alt-N]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','asn')"))

      self.menuBar.addmenuitem('Residue', 'command', 'Arginine',
                               label='Arginine [Alt-R]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','arg')"))

      self.menuBar.addmenuitem('Residue', 'command', 'Cysteine',
                               label='Cysteine [Alt-C]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','cys')"))

      self.menuBar.addmenuitem('Residue', 'command', 'Glutamate',
                               label='Glutamate [Alt-E]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','glu')"))

      self.menuBar.addmenuitem('Residue', 'command', 'Glutamine',
                               label='Glutamine [Alt-N]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','gln')"))

      self.menuBar.addmenuitem('Residue', 'command', 'Glycine',
                               label='Glycine [Alt-G]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','gly')"))

      self.menuBar.addmenuitem('Residue', 'command', 'Histidine',
                               label='Histidine [Alt-H]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','his')"))

      self.menuBar.addmenuitem('Residue', 'command', 'Isoleucine',
                               label='Isoleucine [Alt-I]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','ile')"))

      self.menuBar.addmenuitem('Residue', 'command', 'Leucine',
                               label='Leucine [Alt-L]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','leu')"))

      self.menuBar.addmenuitem('Residue', 'command', 'Lysine',
                               label='Lysine [Alt-K]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','lys')"))

      self.menuBar.addmenuitem('Residue', 'command', 'Methionine',
                               label='Methionine [Alt-M]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','met')"))

      self.menuBar.addmenuitem('Residue', 'command', 'N-Methyl',
                               label='N-Methyl [Alt-Z]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','nme')"))

      self.menuBar.addmenuitem('Residue', 'command', 'Phenylalanine',
                               label='Phenylalanine [Alt-F]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','phe')"))

      self.menuBar.addmenuitem('Residue', 'command', 'Proline',
                               label='Proline [Alt-P]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','pro')"))

      self.menuBar.addmenuitem('Residue', 'command', 'Serine',
                               label='Serine [Alt-S]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','ser')"))

      self.menuBar.addmenuitem('Residue', 'command', 'Threonine',
                               label='Threonine [Alt-T]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','thr')"))

      self.menuBar.addmenuitem('Residue', 'command', 'Tryptophan',
                               label='Tryptophan [Alt-W]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','trp')"))

      self.menuBar.addmenuitem('Residue', 'command', 'Tyrosine',
                               label='Tyrosine [Alt-Y]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','tyr')"))

      self.menuBar.addmenuitem('Residue', 'command', 'Valine',
                               label='Valine [Alt-V]',
                               command = lambda: cmd.do(
         "_ editor.attach_amino_acid('pk1','val')"))


      self.menuBar.addmenuitem('Residue', 'separator', '')
      self.menuBar.addmenuitem('Residue', 'command', 'Helix',
                               label='Helix',
                               command = lambda: cmd.do("_ set secondary_structure,1"))

      self.menuBar.addmenuitem('Residue', 'command', 'Antiparallel Beta Sheet',
                               label='Antiparallel Beta Sheet',
                               command = lambda: cmd.do("_ set secondary_structure,2"))

      self.menuBar.addmenuitem('Residue', 'command', 'Parallel Beta Sheet',
                               label='Parallel Beta Sheet',
                               command = lambda: cmd.do("_ set secondary_structure,3"))

      
      self.menuBar.addmenu('Movie', 'Movie Control')

      self.menuBar.addmenuitem('Movie', 'checkbutton',
                         'Photorealistic images.',
                         label=self.pad+'Ray Trace Frames',
                        variable = self.setting.ray_trace_frames,
                        command = lambda s=self: s.setting.update('ray_trace_frames'))

      self.menuBar.addmenuitem('Movie', 'checkbutton',
                         'Save images in memory.',
                         label=self.pad+'Cache Frames',
                        variable = self.setting.cache_frames,
                        command = lambda s=self: s.setting.update('cache_frames'))

      self.menuBar.addmenuitem('Movie', 'command', 'Flush Image Cache',
                               label=self.pad+'Flush Image Cache',
                               command = lambda: cmd.mclear())

      self.menuBar.addmenuitem('Movie', 'separator', '')

      self.menuBar.addmenuitem('Movie', 'checkbutton',
                         'Static Singleton Objects.',
                         label=self.pad+'Static Singleton Objects',
                        variable = self.setting.static_singletons,
                        command = lambda s=self: s.setting.update('static_singletons'))

      self.menuBar.addmenuitem('Movie', 'checkbutton',
                         'Superimpose all molecular states.',
                         label=self.pad+'Show All States',
                        variable = self.setting.all_states,
                        command = lambda s=self: s.setting.update('all_states'))

      self.menuBar.addmenuitem('Movie', 'separator', '')
      
      self.menuBar.addmenuitem('Movie', 'command', 'Maximum Speed',
                               label=self.pad+'Maximum Speed',
                               command = lambda: cmd.set("movie_delay","0",log=1))

      self.menuBar.addmenuitem('Movie', 'command', '30 FPS',
                               label=self.pad+'30 FPS',
                               command = lambda: cmd.set("movie_delay","33",log=1))

      self.menuBar.addmenuitem('Movie', 'command', '15 FPS',
                               label=self.pad+'15 FPS',
                               command = lambda: cmd.set("movie_delay","66",log=1))

      self.menuBar.addmenuitem('Movie', 'command', '5 FPS',
                               label=self.pad+'5 FPS',
                               command = lambda: cmd.set("movie_delay","200",log=1))

      self.menuBar.addmenuitem('Movie', 'command', '1 FPS',
                               label=self.pad+'1 FPS',
                               command = lambda: cmd.set("movie_delay","1000",log=1))

      self.menuBar.addmenuitem('Movie', 'command', '0.3 FPS',
                               label=self.pad+'0.3 FPS',
                               command = lambda: cmd.set("movie_delay","3000",log=1))

      self.menuBar.addmenuitem('Movie', 'separator', '')

      self.menuBar.addmenuitem('Movie', 'command', 'Reset Meter',
                               label=self.pad+'Reset Meter',
                               command = lambda: cmd.do("_ meter_reset"))

      self.menuBar.addmenu('Display', 'Display Control')


      self.menuBar.addmenuitem('Display', 'command', 'Clear Text Output',
                               label='Clear Text',
                               command = lambda: cmd.do("_ cls"))

      self.menuBar.addmenuitem('Display', 'command', 'Hide Text Output',
                               label='Hide Text',
                               command = lambda: cmd.set("text","0",log=1))

      self.menuBar.addmenuitem('Display', 'command', 'Show Text Output',
                               label='Show Text',
                               command = lambda: cmd.set("text","1",log=1))

      self.menuBar.addmenuitem('Display', 'separator', '')
      
      self.menuBar.addmenuitem('Display', 'command', 'Stereo On',
                               label='Stereo On',
                               command = lambda: cmd.do("_ stereo on"))

      self.menuBar.addmenuitem('Display', 'command', 'Stereo Off',
                               label='Stereo Off',
                               command = lambda: cmd.do("_ stereo off"))

      self.menuBar.addcascademenu('Display', 'Stereo', 'Stereo',
                               label='Stereo')

      self.menuBar.addmenuitem('Stereo', 'command', 'Cross-Eye Stereo',
                               label='Cross-Eye Stereo',
                               command = lambda: cmd.do("_ stereo crosseye"))

      self.menuBar.addmenuitem('Stereo', 'command', 'Quad-Buffered Stereo',
                               label='Quad-Buffered Stereo',
                               command = lambda: cmd.do("_ stereo quadbuffer"))

      self.menuBar.addmenuitem('Stereo', 'separator', '')

      self.menuBar.addmenuitem('Stereo', 'command', 'Swap Sides',
                               label='Swap Sides',
                               command = lambda: cmd.do("_ stereo swap"))

      self.menuBar.addmenuitem('Display', 'separator', '')

      self.menuBar.addmenuitem('Display', 'command', 'Maximum Performance',
                               label='Maximum Performance',
                               command = lambda : cmd.do("_ util.performance(100)"))

      self.menuBar.addmenuitem('Display', 'command', 'Reasonable Performance',
                               label='Reasonable Performance',
                               command = lambda : cmd.do("_ util.performance(66)"))
      
      self.menuBar.addmenuitem('Display', 'command', 'Reasonable Quality',
                               label='Reasonable Quality',
                               command = lambda : cmd.do("_ util.performance(33)"))

      self.menuBar.addmenuitem('Display', 'command', 'Maximum Quality',
                               label='Maximum Quality',
                               command = lambda : cmd.do("_ util.performance(0)"))

      self.menuBar.addmenuitem('Display', 'separator', '')

      self.menuBar.addmenuitem('Display', 'command', 'Light Shadows',
                               label='Light Shadows',
                               command = lambda : cmd.do("_ util.ray_shadows('light')"))

      self.menuBar.addmenuitem('Display', 'command', 'Matte Shadows',
                               label='Matte Shadows',
                               command = lambda : cmd.do("_ util.ray_shadows('matte')"))

      self.menuBar.addmenuitem('Display', 'command', 'Medium Shadows',
                               label='Medium Shadows',
                               command = lambda : cmd.do("_ util.ray_shadows('medium')"))

      self.menuBar.addmenuitem('Display', 'command', 'Heavy Shadows',
                               label='Heavy Shadows',
                               command = lambda : cmd.do("_ util.ray_shadows('heavy')"))

      self.menuBar.addmenuitem('Display', 'command', 'Black Shadows',
                               label='Black Shadows',
                               command = lambda : cmd.do("_ util.ray_shadows('black')"))

      self.menuBar.addmenuitem('Display', 'separator', '')

      self.menuBar.addmenuitem('Display', 'command', 'White Background',
                               label='White Background',
                               command = lambda : cmd.do("_ cmd.bg_color('white')"))

      self.menuBar.addmenuitem('Display', 'command', 'Black Background',
                               label='Black Background',
                               command = lambda : cmd.do("_ cmd.bg_color('black')"))

      
      self.menuBar.addmenu('Setting', 'Configuration Control')

      self.menuBar.addmenuitem('Setting', 'command',
                         'Edit PyMOL Settings',
                         label=self.pad+'Edit All...',
                               command = lambda s=self: SetEditor(s))

      self.menuBar.addmenuitem('Setting', 'command',
                         'Edit PyMOL Colors',
                         label=self.pad+'Colors...',
                               command = lambda s=self: ColorEditor(s))

      self.menuBar.addmenuitem('Setting', 'separator', '')
      
      self.menuBar.addmenuitem('Setting', 'checkbutton',
                         'Show Valences.',
                         label=self.pad+'Show Valences',
                        variable = self.setting.valence,
                        command = lambda s=self: s.setting.update('valence'))

      self.menuBar.addmenuitem('Setting', 'checkbutton',
                         'Disable perspective.',
                         label=self.pad+'Orthoscopic View',
                        variable = self.setting.ortho,
                        command = lambda s=self: s.setting.update('ortho'))

      self.menuBar.addmenuitem('Setting', 'checkbutton',
                         'Smooth Lines.',
                         label=self.pad+'Smooth Lines',
                        variable = self.setting.line_smooth,
                        command = lambda s=self: s.setting.update('line_smooth'))

      self.menuBar.addmenuitem('Setting', 'checkbutton',
                         'Depth Cue Fog.',
                         label=self.pad+'Depth Cue & Ray Trace Fog',
                        variable = self.setting.depth_cue,
                        command = lambda s=self: s.setting.update('depth_cue'))

      self.menuBar.addmenuitem('Setting', 'checkbutton',
                         'Two Sided Lighting.',
                         label=self.pad+'Two Sided Lighting',
                        variable = self.setting.two_sided_lighting,
                        command = lambda s=self: s.setting.update('two_sided_lighting'))

      self.menuBar.addmenuitem('Setting', 'checkbutton',
                         'Specular Reflections.',
                         label=self.pad+'Specular Reflections',
                        variable = self.setting.specular,
                        command = lambda s=self: s.setting.update('specular'))

      self.menuBar.addmenuitem('Setting', 'checkbutton',
                         'Overlay',
                         label=self.pad+'Overlay Text on Graphics',
                        variable = self.setting.overlay,
                        command = lambda s=self: s.setting.update('overlay'))

      self.menuBar.addmenuitem('Setting', 'separator', '')
      
      self.menuBar.addmenuitem('Setting', 'checkbutton',
                         'Smooth raytracing.',
                         label=self.pad+'Antialiased Rendering',
                        variable = self.setting.antialias,
                        command = lambda s=self: s.setting.update('antialias'))

      self.menuBar.addmenuitem('Setting', 'checkbutton',
                         'Cull Backfaces when Rendering',
                         label=self.pad+'Cull Backfaces when Rendering',
                        variable = self.setting.backface_cull,
                        command = lambda s=self: s.setting.update('backface_cull'))

      self.menuBar.addmenuitem('Setting', 'separator', '')

      self.menuBar.addmenuitem('Setting', 'checkbutton',
                               'Ignore PDB segi.',
                               label=self.pad+'Ignore PDB Segment Identifier',
                               variable = self.setting.ignore_pdb_segi,
                               command = lambda s=self: s.setting.update('ignore_pdb_segi'))

      self.menuBar.addmenuitem('Setting', 'checkbutton',
                         'Auto-Zoom.',
                         label=self.pad+'Auto-Zoom New Objects',
                        variable = self.setting.auto_zoom,
                        command = lambda s=self: s.setting.update('auto_zoom'))

      self.menuBar.addmenuitem('Setting', 'checkbutton',
                         'Auto-Show Selections.',
                         label=self.pad+'Auto-Show New Selections',
                        variable = self.setting.auto_show_selections,
                        command = lambda s=self: s.setting.update('auto_show_selections'))

      self.menuBar.addmenuitem('Setting', 'checkbutton',
                         'Auto-Hide Selections.',
                         label=self.pad+'Auto-Hide Selections',
                        variable = self.setting.auto_hide_selections,
                        command = lambda s=self: s.setting.update('auto_hide_selections'))

      self.menuBar.addmenuitem('Setting', 'checkbutton',
                         'Auto-Remove Hydrogens.',
                         label=self.pad+'Auto-Remove Hydrogens',
                        variable = self.setting.auto_remove_hydrogens,
                        command = lambda s=self: s.setting.update('auto_remove_hydrogens'))

      
      self.menuBar.addmenu('Mouse', 'Mouse Configuration')

      self.menuBar.addmenuitem('Mouse', 'command', '3 Button',
                               label='3 Button',
                               command = lambda: cmd.config_mouse('three_button'))

      self.menuBar.addmenuitem('Mouse', 'command', '2 Button',
                               label='2 Button',
                               command = lambda: cmd.config_mouse('two_button'))

      self.menuBar.addmenuitem('Mouse', 'command', '2 Button Editing',
                               label='2 Button Editing',
                               command = lambda: cmd.config_mouse('two_button_editing'))

      self.menuBar.addmenuitem('Mouse', 'separator', '')

      self.menuBar.addmenuitem('Mouse', 'command', '3 Button Viewing',
                               label='3 Button Viewing',
                               command = lambda: cmd.mouse('three_button_viewing'))

      self.menuBar.addmenuitem('Mouse', 'command', '3 Button Editing',
                               label='3 Button Editing',
                               command = lambda: cmd.mouse('three_button_editing'))

      self.menuBar.addmenuitem('Mouse', 'command', '2 Button Viewing',
                               label='2 Button Viewing',
                               command = lambda: cmd.mouse('two_button_viewing'))

      self.menuBar.addmenuitem('Mouse', 'command', '2 Button Selecting',
                               label='2 Button Selecting',
                               command = lambda: cmd.mouse('two_button_selecting'))

      self.menuBar.addmenuitem('Mouse', 'command', '2 Button Editing',
                               label='2 Button Editing',
                               command = lambda: cmd.mouse('two_button_editing'))

      self.menuBar.addmenu('Cartoon', 'Cartoon Properties')

      self.menuBar.addmenuitem('Cartoon', 'checkbutton',
                         'Round Helices',
                         label=self.pad+'Round Helices',
                        variable = self.setting.cartoon_round_helices,
                        command = lambda s=self: s.setting.update('cartoon_round_helices'))

      self.menuBar.addmenuitem('Cartoon', 'checkbutton',
                         'Fancy Helices',
                         label=self.pad+'Fancy Helices',
                        variable = self.setting.cartoon_fancy_helices,
                        command = lambda s=self: s.setting.update('cartoon_fancy_helices'))

      self.menuBar.addmenuitem('Cartoon', 'checkbutton',
                         'Cylindrical Helices',
                         label=self.pad+'Cylindrical Helices',
                        variable = self.setting.cartoon_cylindrical_helices,
                        command = lambda s=self: s.setting.update('cartoon_cylindrical_helices'))

      self.menuBar.addmenuitem('Cartoon', 'checkbutton',
                         'Flat Sheets',
                         label=self.pad+'Flat Sheets',
                        variable = self.setting.cartoon_flat_sheets,
                        command = lambda s=self: s.setting.update('cartoon_flat_sheets'))


      self.menuBar.addmenuitem('Cartoon', 'checkbutton',
                         'Fancy Sheets',
                         label=self.pad+'Fancy Sheets',
                        variable = self.setting.cartoon_fancy_sheets,
                        command = lambda s=self: s.setting.update('cartoon_fancy_sheets'))

      self.menuBar.addmenuitem('Cartoon', 'checkbutton',
                         'Smooth Loops',
                         label=self.pad+'Smooth Loops',
                        variable = self.setting.cartoon_smooth_loops,
                        command = lambda s=self: s.setting.update('cartoon_smooth_loops'))

      self.menuBar.addmenuitem('Cartoon', 'checkbutton',
                         'Discrete Colors',
                         label=self.pad+'Discrete Colors',
                        variable = self.setting.cartoon_discrete_colors,
                        command = lambda s=self: s.setting.update('cartoon_discrete_colors'))


      self.menuBar.addmenu('Wizard', 'Task Wizards')
      
      self.menuBar.addmenuitem('Wizard', 'command', 'Density Map Wizard',
                               label='Density',
                               command = lambda: cmd.do("_ wizard density"))

      self.menuBar.addmenuitem('Wizard', 'command', 'Distance',
                               label='Distance',
                               command = lambda: cmd.do("_ wizard distance"))

      self.menuBar.addmenuitem('Wizard', 'command', 'Filter',
                               label='Filter',
                               command = lambda: cmd.do("_ wizard filter"))

      self.menuBar.addmenuitem('Wizard', 'command', 'Mutagenesis',
                               label='Mutagenesis',
                               command = lambda: cmd.do("_ wizard mutagenesis"))

      self.menuBar.addmenuitem('Wizard', 'command', 'Pair Fitting',
                               label='Pair Fitting',
                               command = lambda: cmd.do("_ wizard pair_fit"))

      self.menuBar.addmenuitem('Wizard', 'command', 'Sculpting',
                               label='Sculpting',
                               command = lambda: cmd.do("_ wizard sculpting"))

      self.menuBar.addmenuitem('Wizard', 'separator', '')
      
      self.menuBar.addmenuitem('Wizard', 'command', 'Label',
                               label='Label',
                               command = lambda: cmd.do("_ wizard label"))

      self.menuBar.addmenuitem('Wizard', 'command', 'Charge',
                               label='Charge',
                               command = lambda: cmd.do("_ wizard charge"))


      self.menuBar.addmenu('Sculpting', 'Sculpting')

      self.menuBar.addmenuitem('Sculpting', 'checkbutton',
                         'Auto-Sculpt.',
                         label=self.pad+'Auto-Sculpting',
                        variable = self.setting.auto_sculpt,
                        command = lambda s=self: s.setting.update('auto_sculpt'))

      self.menuBar.addmenuitem('Sculpting', 'checkbutton',
                         'Sculpting.',
                         label=self.pad+'Sculpting',
                        variable = self.setting.sculpting,
                        command = lambda s=self: s.setting.update('sculpting'))

      self.menuBar.addmenuitem('Sculpting', 'separator', '')
      

      self.menuBar.addmenuitem('Sculpting', 'command', 'Activate',
                               label='Activate',
                               command = lambda: cmd.do("_ sculpt_activate all"))

      self.menuBar.addmenuitem('Sculpting', 'command', 'Deactivate',
                               label='Deactivate',
                               command = lambda: cmd.do("_ sculpt_deactivate all"))

      self.menuBar.addmenuitem('Sculpting', 'command', 'Clear Memory',
                               label='Clear Memory',
                               command = lambda: cmd.do("_ sculpt_purge"))

      self.menuBar.addmenuitem('Sculpting', 'separator', '')

      self.menuBar.addmenuitem('Sculpting', 'command', '1 Cycle/Update',
                               label='1 Cycle per Update',
                               command = lambda: cmd.do("_ set sculpting_cycles=1"))

      self.menuBar.addmenuitem('Sculpting', 'command', '3 Cycles/Update',
                               label='3 Cycles per Update',
                               command = lambda: cmd.do("_ set sculpting_cycles=3"))

      self.menuBar.addmenuitem('Sculpting', 'command', '10 Cycles/Update',
                               label='10 Cycles per Update',
                               command = lambda: cmd.do("_ set sculpting_cycles=10"))

      self.menuBar.addmenuitem('Sculpting', 'command', '33 Cycles/Update',
                               label='33 Cycles per Update',
                               command = lambda: cmd.do("_ set sculpting_cycles=33"))

      self.menuBar.addmenuitem('Sculpting', 'command', '100 Cycles/Update',
                               label='100 Cycles per Update',
                               command = lambda: cmd.do("_ set sculpting_cycles=100"))

      self.menuBar.addmenuitem('Sculpting', 'command', '333 Cycles/Update',
                               label='333 Cycles per Update',
                               command = lambda: cmd.do("_ set sculpting_cycles=333"))

      self.menuBar.addmenuitem('Sculpting', 'command', '1000 Cycles/Update',
                               label='1000 Cycles per Update',
                               command = lambda: cmd.do("_ set sculpting_cycles=1000"))

      self.menuBar.addmenuitem('Sculpting', 'separator', '')

      self.menuBar.addmenuitem('Sculpting', 'command', 'Bonds Only',
                               label='Bonds Only',
                               command = lambda: cmd.do("_ set sculpt_field_mask=1"))

      self.menuBar.addmenuitem('Sculpting', 'command', 'Bonds & Angles Only',
                               label='Bonds & Angles Only',
                               command = lambda: cmd.do("_ set sculpt_field_mask=3"))

      self.menuBar.addmenuitem('Sculpting', 'command', 'All Except VDW',
                               label='All Except VDW',
                               command = lambda: cmd.do("_ set sculpt_field_mask=31"))

      self.menuBar.addmenuitem('Sculpting', 'command', 'All Except 1-4 VDW',
                               label='All Except 1-4 VDW',
                               command = lambda: cmd.do("_ set sculpt_field_mask=63"))

      self.menuBar.addmenuitem('Sculpting', 'command', 'All Terms',
                               label='All Terms',
                               command = lambda: cmd.do("_ set sculpt_field_mask=127"))

      self.menuBar.addmenu('Demo', 'Demonstrations')

      self.demo = Demo()
      
      self.menuBar.addmenuitem('Demo', 'command', 'Representations',
                               label='Representations',
                               command = lambda s=self:s.demo('rep'))

      self.menuBar.addmenuitem('Demo', 'command', 'Cartoon Ribbons',
                               label='Cartoon Ribbons',
                               command = lambda s=self:s.demo('cartoon'))

      self.menuBar.addmenuitem('Demo', 'command', 'Transparency',
                               label='Transparency',
                               command = lambda s=self:s.demo('trans'))

      self.menuBar.addmenuitem('Demo', 'command', 'Ray Tracing',
                               label='Ray Tracing',
                               command = lambda s=self:s.demo('ray'))

      self.menuBar.addmenuitem('Demo', 'command', 'Sculpting',
                               label='Sculpting',
                               command = lambda s=self:s.demo('sculpt'))

      self.menuBar.addmenuitem('Demo', 'command', 'Scripted Animation',
                               label='Scripted Animation',
                               command = lambda s=self:s.demo('anime'))

      self.menuBar.addmenuitem('Demo', 'command', 'Molscript/Raster3D Input',
                               label='Molscript/Raster3D Input',
                               command = lambda s=self:s.demo('raster3d'))

      self.menuBar.addmenuitem('Demo', 'command', 'Compiled Graphics Objects',
                               label='Compiled Graphics Objects',
                               command = lambda s=self:s.demo('cgo'))




from Numeric import *

class Brick:
   
   def __init__(self):
      self.valid = None

   def setup_from_min_max(self,mn,mx,grid,buffer=0.0):
      self.origin = [
         mn[0]-buffer,
         mn[1]-buffer,
         mn[2]-buffer
         ]
      self.range = [
         (mx[0]-mn[0])+2*buffer,
         (mx[1]-mn[1])+2*buffer,
         (mx[2]-mn[2])+2*buffer
         ]
      self.dim = [
         1 + int(self.range[0]/grid[0]),
         1 + int(self.range[1]/grid[1]),
         1 + int(self.range[2]/grid[2])
         ]
      self.grid = copy.deepcopy(grid)
      self.lvl = zeros(self.dim,Float)
      
      




#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------

import cmd
from cmd import _cmd,lock,unlock
import selector
import os

from cmd import _cmd,lock,unlock,Shortcut,QuietException

def align(source,target,cutoff=2.0,cycles=2,gap=-10.0,extend=-0.5,
          skip=0,object=None,matrix="BLOSUM62",quiet=1): 
   '''
DESCRIPTION
  
   "align" performs a sequence alignment followed by a structural
   alignment, and then carrys out zero or more cycles of refinement
   in order to reject structural outliers found during the fit.

USAGE 
 
   align (source), (target) [,cutoff [,cycles [,gap [,extend \\
         [,skip [,object [,matrix [, quiet ]]]]]]]]
 
PYMOL API
  
   cmd.align( string source, string target, float cutoff=2.0,
              int cycles=2, float gap=-10.0, float extend=-0.5,
              float extend=-0.5,int skip=0, string object=None,
              string matrix="BLOSUM62",int quiet=1 )

NOTE

   If object is not None, then align will create an object which
   indicates which atoms were paired between the two structures

EXAMPLES
 
   align  prot1////CA, prot2, object=alignment
   
SEE ALSO

   fit, rms, rms_cur, intra_rms, intra_rms_cur, pair_fit
   '''
   r = None
   source = selector.process(source)
   target = selector.process(target)
   mfile = os.path.expandvars("$PYMOL_PATH/modules/pymol/matrices/"+matrix)
   if object==None: object=''
   try:
      lock()
      r = _cmd.align(source,target,float(cutoff),int(cycles),float(gap),
                     float(extend),int(skip),str(object),str(mfile),
                     int(quiet))
   finally:
      unlock()
   return r

def intra_fit(selection,state=1,quiet=1):
   '''
DESCRIPTION
  
   "intra_fit" fits all states of an object to an atom selection
   in the specified state.  It returns the rms values to python
   as an array.
      
USAGE 
 
   intra_fit (selection),state
 
PYMOL API
  
   cmd.intra_fit( string selection, int state )
    
EXAMPLES
 
   intra_fit ( name ca )
   
PYTHON EXAMPLE
 
   from pymol import cmd
   rms = cmd.intra_fit("(name ca)",1)

SEE ALSO

   fit, rms, rms_cur, intra_rms, intra_rms_cur, pair_fit
   '''
   # preprocess selection
   selection = selector.process(selection)
   #   
   r = None
   state = int(state)
   try:
      lock()
      r = _cmd.intrafit("("+str(selection)+")",int(state)-1,2,int(quiet))
   finally:
      unlock()
   if not r:
      if cmd._raising(): raise QuietException
   elif not quiet:
      st = 1
      for a in r:
         if a>=0.0:
            print " cmd.intra_fit: %5.3f in state %d vs state %d"%(a,st,state)
         st = st + 1
   return r

def intra_rms(selection,state=0,quiet=1):
   '''
DESCRIPTION
  
   "intra_rms" calculates rms fit values for all states of an object
   over an atom selection relative to the indicated state.  
   Coordinates are left unchanged.  The rms values are returned
   as a python array.
      
PYMOL API
 
   cmd.intra_rms( string selection, int state)
 
PYTHON EXAMPLE
 
   from pymol import cmd
   rms = cmd.intra_rms("(name ca)",1)

SEE ALSO

   fit, rms, rms_cur, intra_fit, intra_rms_cur, pair_fit
   '''
   # preprocess selection
   selection = selector.process(selection)
   #   
   r = None
   state = int(state)
   try:
      lock()
      r = _cmd.intrafit("("+str(selection)+")",int(state)-1,1,int(quiet))
   finally:
      unlock()
   if not r:
      if cmd._raising(): raise QuietException
   elif not quiet:
      st = 1
      for a in r:
         if a>=0.0:
            print " cmd.intra_rms: %5.3f in state %d vs state %d"%(a,st,state)
         st = st + 1
   return r

def intra_rms_cur(selection,state=0,quiet=1):
   '''
DESCRIPTION
  
   "intra_rms_cur" calculates rms values for all states of an object
   over an atom selection relative to the indicated state without
   performing any fitting.  The rms values are returned
   as a python array.
      
PYMOL API
 
   cmd.intra_rms_cur( string selection, int state)
 
PYTHON EXAMPLE
 
   from pymol import cmd
   rms = cmd.intra_rms_cur("(name ca)",1)

SEE ALSO

   fit, rms, rms_cur, intra_fit, intra_rms, pair_fit
   '''
   # preprocess selection
   selection = selector.process(selection)
   #   
   r = None
   state = int(state)
   try:
      lock()
      r = _cmd.intrafit("("+str(selection)+")",int(state)-1,0,int(quiet))
   finally:
      unlock()
   if not r:
      if cmd._raising(): raise QuietException
   elif not quiet:
      st = 1
      for a in r:
         if a>=0.0:
            print " cmd.intra_rms_cur: %5.3f in state %d vs state %d"%(a,st,state)
         st = st + 1
   return r

def fit(selection,target,quiet=1):
   '''
DESCRIPTION
  
   "fit" superimposes the model in the first selection on to the model
   in the second selection.  Only matching atoms in both selections
   will be used for the fit.
   
USAGE
 
   fit (selection), (target-selection)
 
EXAMPLES
 
   fit ( mutant and name ca ), ( wildtype and name ca )

SEE ALSO

   rms, rms_cur, intra_fit, intra_rms, intra_rms_cur
   '''
   a=str(selection)
   b=str(target)
   # preprocess selections
   a = selector.process(a)
   b = selector.process(b)
   #
   try:
      lock()   
      r = _cmd.fit("((%s) in (%s))" % (str(a),str(b)),
                  "((%s) in (%s))" % (str(b),str(a)),2,int(quiet))
   finally:
      unlock()
   return r

def rms(selection,target,quiet=1):
   '''
DESCRIPTION
  
   "rms" computes a RMS fit between two atom selections, but does not
   tranform the models after performing the fit.
   
USAGE
 
   rms (selection), (target-selection)
 
EXAMPLES
 
   fit ( mutant and name ca ), ( wildtype and name ca )

SEE ALSO

   fit, rms_cur, intra_fit, intra_rms, intra_rms_cur, pair_fit   
   '''
   a=str(selection)
   b=str(target)
   # preprocess selections
   a = selector.process(a)
   b = selector.process(b)
   #
   try:
      lock()   
      r = _cmd.fit("((%s) in (%s))" % (str(a),str(b)),
                  "((%s) in (%s))" % (str(b),str(a)),1,int(quiet))
   finally:
      unlock()
   return r

def rms_cur(selection,target,quiet=1):
   '''
DESCRIPTION
  
   "rms_cur" computes the RMS difference between two atom
   selections without performing any fitting.
   
USAGE
 
   rms_cur (selection), (selection)

SEE ALSO

   fit, rms, intra_fit, intra_rms, intra_rms_cur, pair_fit   
   '''
   a=str(selection)
   b=str(target)
   # preprocess selections
   a = selector.process(a)
   b = selector.process(b)
   #   
   try:
      lock()   
      r = _cmd.fit("((%s) in (%s))" % (str(a),str(b)),
                  "((%s) in (%s))" % (str(b),str(a)),0,int(quiet))
   finally:
      unlock()
   return r

def pair_fit(*arg):
   '''
DESCRIPTION
  
   "pair_fit" fits a set of atom pairs between two models.  Each atom
   in each pair must be specified individually, which can be tedious
   to enter manually.  Script files are recommended when using this
   command.
   
USAGE
 
   pair_fit (selection), (selection), [ (selection), (selection) [ ...] ]

SEE ALSO

   fit, rms, rms_cur, intra_fit, intra_rms, intra_rms_cur
   '''
   new_arg = []
   for a in arg:
      new_arg.append(selector.process(a))
   try:
      lock()   
      r = _cmd.fit_pairs(new_arg)
   finally:
      unlock()
   return r







import string
import re
import copy

from chempy import io

class SDFRec:
   
   def __init__(self,sdflist):
      getkee = re.compile("^>\s+<([^>]*)>")
      gettag = re.compile("^>\s+<[^>]*>\s+\((.*)\)")
      ll = len(sdflist)
      if ll<4:
         print " SDFRec: invalid SDF record format #1"
         return None
      self.kees = ['MOL'] # separate key list to preserve order
      self.data = {}
      self.ref_code = {}
      self.data['MOL'] = []
      mol = self.data['MOL']
      l = 0
      while l<ll:
         mol.append(sdflist[l])
         if sdflist[l][0:6]=='M  END':
            break;
         if sdflist[l][0:1]=='>':
            mol[len(mol)-1]='M  END\n'
            sdflist.insert(l,'M  END\n')
            ll=len(sdflist)
            break;
         l = l + 1
         if l>=ll:
            print " SDFRec: invalid SDF record format #2"
            return None            
      while l<ll:
         if sdflist[l][0]=='>':
            sl = sdflist[l]
            kee_match = getkee.match(sl)
            if not kee_match:
               print " SDFRec: invalid SDF record format #3"
               return None
            kee = kee_match.group(1)
            self.kees.append(kee)
            ref_code_match = gettag.match(sl)
            if ref_code_match:
               self.ref_code[kee] = ref_code_match.group(1)
            else:
               self.ref_code[kee] = ''
            self.data[kee] = []
            sd = self.data[kee]
            l = l + 1
            while l<ll:
               if len(string.strip(sdflist[l]))!=0:
                  sd.append(sdflist[l])
                  l = l + 1
               else:
                  break;
         else:
            l = l + 1

   def toList(self):
      r = []
      for k in self.kees:
         if k!='MOL':
            if self.ref_code[k]!='':
               r.append(">  <"+k+"> ("+self.ref_code[k]+")\n")
            else:
               r.append(">  <"+k+">\n")               
         for a in self.data[k]:
            r.append(a)
         if k!='MOL':
            r.append("\n")
      return r

   def get(self,kee):
      if self.data.has_key(kee):
         return self.data[kee]
      else:
         return None

   def get_single(self,kee): # automatic stripping
      if self.data.has_key(kee):
         sdk = self.data[kee]
         if len(sdk):
            return string.strip(sdk[0])
         else:
            return None
      else:
         return None
         
   def set_single(self,kee,data,ref_code=None): # adds LF
      self.set(kee,[data+'\n'],ref_code)
      
   def get_model(self):
      return io.mol.fromList(self.get('MOL'))
      
   def set_model(self,model):
      self.set('MOL',io.mol.toList(model))
               
   def set(self,kee,data,ref_code=None):
      if kee not in self.kees:
         self.kees.append(kee)
         self.ref_code[kee]=''
      if ref_code!=None:
         self.ref_code[kee]=ref_code
      self.data[kee] = copy.deepcopy(data)

   def delete(self,kee):
      self.kees.remove(kee)
      del self.data[kee]

class SDF:
   
   def __init__(*args):
      mode = 'r'
      if len(args)<2:
         raise ValueError
      self = args[0]
      fname = args[1]
      if len(args)==3:
         mode = args[2]
      self.mode = mode
      self.at_eof = 0
      if mode not in ('w','r','wa'):
         print " SDF: bad mode"
         return None
      self.file = open(fname,mode)

   def write(self,rec):
      lst = rec.toList()
      for a in lst:
         self.file.write(a)
      self.file.write('$$$$\n')
      
   def read(self): # returns SDFRec or None at end of file
      cur = []
      while 1:
         s = self.file.readline()
         if not s:
            return None
         elif s[0:4]==r'$$$$':
            return SDFRec(cur)
         else:
            cur.append(s)
         
   def close(self):
      self.file.close()
      



